package com.americancoders.edi;

import java.util.*;
import com.americancoders.edi.*;
import org.w3c.dom.*;

/**
 *   class for all Composite Data Elements
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class CompositeDE implements IDataElementContainer
 {
    protected String id;
    protected Object deArray[];
    protected String xmlTag;
    protected TemplateComposite myTemplate = null;
    protected int sequence;

 /** creates a composite with an id, sequence and a xmltag
  * @param String id
  * @param int sequence
  * @param String tag
  */
   public CompositeDE(String inID, int inSequence, String inXMLTag)
      {
        id = inID;
        sequence = inSequence;
        deArray = null;
        setXMLTag(inXMLTag);
      }  
      
  /** creates a compsosite using a templage
  * @param templatecomposite
  * 
  */ 
  public CompositeDE(TemplateComposite inTemplateComposite)
         {
           id = inTemplateComposite.getID();
           sequence = inTemplateComposite.getSequence();
           setXMLTag(inTemplateComposite.getXMLTag());
           deArray = new Object[inTemplateComposite.getTemplateDESize()];
           myTemplate = inTemplateComposite;
         }  
      
	  
/** creates a compsosite with an id 
  * @param String id
  */
  
    public CompositeDE(String inID)
      {
        id = inID;
        deArray = null;
      }  
      
     private void testArraySize(int pos)
     {
        
         if (deArray == null)
            if (myTemplate == null || pos > myTemplate.getTemplateDESize())
               deArray = new Object[pos+1];
            else 
                deArray = new Object[myTemplate.getTemplateDESize()];
         else 
            if (pos >= deArray.length) {// rebuild array
              Object tempArray[] = new Object[deArray.length];
              java.lang.System.arraycopy(deArray, 0, tempArray, 0, deArray.length);
              deArray = new Object[pos+1];
              java.lang.System.arraycopy(tempArray, 0, deArray, 0, tempArray.length);
            }  
     }
     
      
      
 /**
  * defines a dataelement by the predefined templateDE array
  * @param int position in templateDE array
  * @exception Exception thrown by data element instantiation.
  */ 
   public void buildDE(int pos)
     {
         

         testArraySize(pos);
         
         TemplateDE tde;
         TemplateComposite tComposite;
         tde = myTemplate.getTemplateDE(pos);
         
         
           switch (tde.getType().charAt(0))
           { 
             case 'A':
               if (tde.getIDList()!= null)
                  deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               else 
               deArray[pos] = new CharDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'C':
               if (tde.getIDList()!= null)
                  deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               else 
               deArray[pos] = new CharDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'I':
               deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               break;
             case 'D':
               deArray[pos] = new DateDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'T':
               deArray[pos] = new TimeDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'N':
               deArray[pos] = new NumericDE(tde.getID(), tde.getSequence(), tde.getType(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'R':
               deArray[pos] = new RealDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             default: 
               throw new OBOEException("Unknown dataElement type");
            }  
      return;
    }    


 /** sets xmltag field
  * @param String
  */
      
 public void  setXMLTag(String inXMLTag)
 {
    xmlTag = inXMLTag;
 }
 
 /**
  * returns the id field 
  * @return String id value
  */
  
  
 public String getID()
 {
    return id;
 }
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
  
 public String getXMLTag()
 {
    if (xmlTag == null) return getID();
    return xmlTag;
 }
 
 /**
  * sets the sequence 
  * @param int
  */
 public void setSequence(int inSequence) {sequence = inSequence;}
 
 /**
  * returns the sequence 
  * @return int
  */
  
 public int getSequence() {return sequence;}
 
/**
 * returns the number of defined data element 
 * @return int DataElement count
*/

public int  getDataElementSize() { return deArray.length; }


/**
 * returns a data element by its id
 *
 * @param String  id of the data element
 * @return DataElement
*/

public DataElement getDataElement(String inID)
  {
      int i;
      DataElement currentDE = null;
      
      for (i=0; i < deArray.length; i++)
        {
             currentDE = (DataElement) deArray[i];
             if (currentDE == null) continue;
             if (currentDE.getID().compareTo(inID) == 0)
                break;  
        } 
      return currentDE;
	}
       
/**
 * returns a data element by its sequence number, not location within vector
 *
 * @param int  sequence number of data element
 * @return DataElement
*/

public DataElement getDataElement(int inSequence)
  {
      if (inSequence >= deArray.length) return null;  
      if (deArray[inSequence] == null) return null;
      return (DataElement) deArray[inSequence];
      
	}
     
/**
 * sets a DataElement in the Container
 *
 * @param String  id of the DataElement
 * @return DataElement  DataElement object
*/
public void addDataElement(DataElement dataElementObject)
    {

       testArraySize(dataElementObject.getSequence()-1);
       deArray[dataElementObject.getSequence()-1] = dataElementObject;
    }    

/** parsing logic for compositeDE
  * @param ITokenizer any tokenizable object
  * 
  */
  
public boolean parse(ITokenizer TransactionTokenizedString)
                throws Exception
   {
      int i;
      int elementsToParse;
      
      DataElement currentDE;
       
      IDataTokenizer dt = TransactionTokenizedString.makeSubfieldTokenizer();
      
      elementsToParse = dt.countTokens();
      for (i=0; 
           (i < elementsToParse);
              i++)
        {
              if (myTemplate != null  && i >= myTemplate.getTemplateDESize())
                return false;
              if (i >= deArray.length || deArray[i] == null)
                  buildDE(i);
              currentDE = (DataElement) deArray[i];
              currentDE.set(dt.nextToken());
        }        
    return true;
   }
/**
 * parses a XML EDI Document
 * adds to datalement vector
 * @param Node input XML node
 *
*/

public void parse(Node node)
                throws Exception
   {
      int i, n, cn;
      int elementsToParse;
      
      NodeList nl = node.getChildNodes();
      NodeList cnl;
      DataElement currentDE;
      CompositeDE currentComposite;
      Node currentNode;
      
      Node cnode;
      int lastDETested = -1;
      nodeLoop: for (n = 0; n < nl.getLength(); n++)
        {
          currentNode = nl.item(n);
          if (currentNode.getNodeType() != Node.ELEMENT_NODE)  continue nodeLoop;
          cnl = currentNode.getChildNodes();
           i = myTemplate.doYouUseThisXMLElement(currentNode.getNodeName(), lastDETested); 
           if (i < 0)
               throw new OBOEException("Unknown node found " +  currentNode.getNodeName());
             if (i > lastDETested)
               {
                for (cn = lastDETested+1; cn <= i; cn++)
                  buildDE(cn); 
               }   
              lastDETested = i;  
              currentDE = (DataElement) deArray[i];
              if (currentDE.getXMLTag().compareTo(currentNode.getNodeName()) == 0)
                  {
                   for (int nn = 0; nn < cnl.getLength(); nn++)
                      {
	                    cnode = cnl.item(nn);
	                    if (cnode.getNodeType() == Node.TEXT_NODE)
                            currentDE.set(cnode.getNodeValue());
	                  }  
                      continue nodeLoop; 
                    } 
           }
   }
   
/**
 * returns all the data associated 
 * @return String for all DataElements 
*/
public String get()
  { 
            if (deArray == null) return "";
            int i;
            StringBuffer sbGet = new StringBuffer();
            DataElement de;
            for (i = 0; i < deArray.length && deArray[i] != null; i++)
              {
               de = (DataElement)deArray[i]; 
               if (de.get() == null) break;
               sbGet.append(de.get());
               sbGet.append('*');
              }
            return new String(sbGet);   
 }     
/**
 * returns all the data associated 
 * @param char separator character
 * @return String for all Dataelements 
 * @exception Exception thrown when data is missing from a data element
*/
public String get(char separator)
          throws Exception
  { 
            if (deArray == null) return "";
            int i;
            StringBuffer sbGet = new StringBuffer();
            for (i = 1; i < deArray.length && deArray[i] != null; i++)
              {
               sbGet.append(((DataElement)deArray[i]).get());
               sbGet.append(separator);
              }
            if (i < 1)
               throw new OBOEException("No known data elements");
            return new String(sbGet);   
 }     
 
 /**
 * returns the formatted text
 * <ul>
 * <li>type == 1 returns XML format
 * <li>otherwise "description: value"
 * </ul>
 */
 
public String getFormattedText(int formatType)
    {
      if (deArray==null) return "";
      if (deArray[0]==null) return "";
      int i;
      DataElement currentDE;
      StringBuffer sbFormattedText = new StringBuffer();
     
      switch (formatType)
           {
               case TransactionSet.XML_FORMAT:
                   if (getXMLTag() != null)
                      sbFormattedText.append('<' +  getXMLTag() + ">\n");
                   else  
                      sbFormattedText.append('<' +  getID() + ">\n");
                  break;
               case TransactionSet.X12_FORMAT:
                   break;
               case TransactionSet.EDIFACT_FORMAT:
                   break;
               default:
                   sbFormattedText.append(getID() + ": ");
           }     
             
      for (i=0; i < deArray.length && deArray[i] != null; i++)
        {
             currentDE = (DataElement) deArray[i];
             if (currentDE.get() == null)
                break;  
              sbFormattedText.append(currentDE.getFormattedText(formatType));
              if (i < deArray.length - 1 && deArray[i+1] != null)
                switch (formatType) {
                    case TransactionSet.X12_FORMAT:
                     sbFormattedText.append(TransactionSet.X12_GROUP_DELIMITER);
                     break;
                    case  TransactionSet.EDIFACT_FORMAT:
                     sbFormattedText.append(TransactionSet.EDIFACT_GROUP_DELIMITER);
                     break;
                  }  
                
        } 
		
      switch (formatType)
        {
         case TransactionSet.XML_FORMAT:
              if (getXMLTag() != null)
               sbFormattedText.append("</" +  getXMLTag() + ">\n");
             else  
               sbFormattedText.append("</" +  getID() + ">\n");
             break;
         case TransactionSet.X12_FORMAT:
             break;
         case TransactionSet.EDIFACT_FORMAT:
             break;
         default:
             break;
        }    
         
      return new String(sbFormattedText);
  
     
    }
    

 
 
 }
