package com.americancoders.edi;

import java.util.*;

/**
 * class for all Data Elements
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class DataElement
 {
    protected int sequence;
    protected String id = "";
    protected String referenceId = "";
    protected int minLength, maxLength;
    protected String value = null;
    protected String description;
    protected String xmlTag;
    protected char required = 'O';
   

/**
 * constructs the Data Element type
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Description
 * @param char required indicator
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
 *
*/
public DataElement(String inID, int inSequence, String inDescription, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
{
  setID(inID);
  setSequence(inSequence);
  setDescription(inDescription);
  setRequired(inRequired);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
  setXMLTag(inXMLTag);
  }

/**
 * constructs the Data Element type
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Desctiption
 * @param char required indicator
 * @param int minimum allowed length
 * @param int maximum allowed length
 *
*/
public DataElement(String inID, int inSequence, String inShortDescription,  char inRequired, int inMinLength, int inMaxLength)
{
  setID(inID);
  setSequence(inSequence);
  setDescription(inShortDescription);
  setRequired(inRequired);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
  setXMLTag(inShortDescription.replace(' ','_'));
 }

/**
 * constructs the Data Element type and sets its value
 * @param String input value
 * @param int sequence number within structure
 * @param String Description
 * @param char required indicator
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
 *
*/
public DataElement(String inValue, String inID, int inSequence, String inDescription, char inRequired, int inMinLength, int inMaxLength)
{
  set(inValue); 
  setID(inID);
  setSequence(inSequence);
  setDescription(inDescription);
  setRequired(inRequired);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
}


/**
 * constructs the Data Element type and sets its value
 * constructs the Data Element type and sets its value
 * @param String input value
 * @param int sequence number within structure
 * @param String Description
 * @param char required indicator
 * @param int minimum allowed length
 * @param int maximum allowed length
 *
*/
public DataElement(String inValue, String inID, int inSequence, String inDescription, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
{
  set(inValue); 
  setID(inID);
  setSequence(inSequence);
  setDescription(inDescription);
  setRequired(inRequired);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
  setXMLTag(inXMLTag);
}


/**
 * returns the dataElement Type
 * @return String type
 *
*/

   public String getType(){return "";} // is there a default type?
   
   
/**
 * sets the Data Element id
 * @param String edi id
 *
*/
    public void setID(String inID) { id = inID; }

/**
 * gets the Data Element id
 * @return String edi id
 *
*/
    public String getID() { return id; }

/**
 * sets the Data Element sequence
 * @param int sequence with in the segment
 *
*/
    public void setSequence(int inSequence) { sequence = inSequence; }

/**
 * gets the Data Element sequence
 * @return int sequence with in the segment
 *
*/
    public int getSequence() { return sequence; }

/**
 * sets the value for the Data Element
 * @param String value
 *
*/
    public void setValue(String inValue) {value = inValue;};

/**
 * sets the required indicator
 * @param char sets the required indicator
 *
*/
    public void setRequired(char inRequired) { required = inRequired; }
/**
 * sets the minimum length for the Data Element
 * @param int sets the minimum length 
 *
*/
    public void setMinLength(int inMinLength) { minLength = inMinLength; }
/**
 * sets the maximum length for the Data Element
 * @param int sets the maximum length 
 *
*/
    public void setMaxLength(int inMaxLength) { maxLength = inMaxLength;  }
/**
 * gets the current length for the Data Element
 * @int the defined minimum length
 *
*/
/**
 * gets the required indicator
 * @return char the required indicator
 *
*/
    public int getRequired() { return required; }
/**
 * gets the minimum length for the Data Element
 * @return int the defined minimum length
 *
*/
    public int getMinLength() { return minLength; }
/**
 * gets the maximum length for the Data Element
 * @return int the defined maximum length
 *
*/
    public int getMaxLength() { return maxLength;  }
/**
 * gets the current length for the Data Element
 * @return int retuns length of set value,  can have a null exception if value is not set.
 *
*/
    public int getLength()    { return value.length(); }

/**
 * sets value for the Data Element
 * @param String value to be set
 *
*/
/** sets the xml tag field 
 * @param String xml tag value
 */

public void  setXMLTag(String inXMLTag){xmlTag = inXMLTag;}
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
public String getXMLTag() { return xmlTag; }
 
  
    public void set(String inValue) {value = new String(inValue);};

/**
 * returns the value for the Data Element
 * @return String
*/

    public String get() {return value;};

/**
 * sets Description for the Data Element
 * @param String description
 *
*/
    public void setDescription(String inDescription) {description = inDescription;};

/**
 * returns the Description for the Data Element
 * @return String description
*/
    public String getDescription() 
      {
        if (description == null || description.length() == 0)
          return id;
        return description;
      };

 /**
  * formats text of data element                     
  * <br> Description of DataElement is defined in the class
  * <br> value is the current value set in the object
  * @param int formatType
  * <ul>
  * <li> formatType == 1 produces XML output
  * <li> default produces "description: value"
  * </ul> 
  */

 public String getFormattedText(int formatType)
    {
      StringBuffer sb = new StringBuffer();
      switch (formatType)
        {
         case TransactionSet.XML_FORMAT:
           if (formatType == TransactionSet.XML_FORMAT)
                sb.append("<" +getXMLTag() + ">" + value + "</" + getXMLTag() + ">\n");
            break;
         case TransactionSet.X12_FORMAT:
             return value;
         case TransactionSet.EDIFACT_FORMAT:
              return value;
         default:
             sb.append("\t" + getDescription() + ": " + value + "\n");
             break;
        }     
       return new String(sb);
    }
 }
