package com.americancoders.edi;

import java.util.*;
import java.text.*;

/**
 * class for Data Elements defined as Date
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class DateDE extends DataElement
 {
    private Calendar calendar;

/**
 * constructs a DateDE object
 * @param String x12 id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
*/
    public DateDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
      {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        calendar = Calendar.getInstance();
      }

/**
 * constructs a DateDE object
 * @param String x12 id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
    public DateDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
      {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        calendar = Calendar.getInstance();
      }

/**
 * constructs a DateDE object using an input string in the format YYYYMMDD where
 * <UL> 
 * <LI> YY is year
 * <LI> MM is month, 
 * <LI> DD is day of month
 *</UL>
 * @param String x12 id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param inDate              String field in format YYYYMMDD
*/
    public DateDE(String inDate, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
      {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        calendar = Calendar.getInstance();
        set(inDate);
      }

/**
 * returns the date stored in YYYYMMDD format  where
 * <UL> 
 * <LI> YY is year
 * <LI> MM is month, 
 * <LI> DD is day of month
 *</UL>

 * @return String in format YYYYMMDD
*/
    public String get()
      {

        if (value == null)
          return null;
          
        if (value.length() == 0) 
          return "";

       SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
       value = sdf.format(calendar.getTime());
       return value;
      }

/**
 * returns the date field as stored
 * @param int field identifier
 * <BR>use the java.util.Calendar fields:
 * <OL>
 * <LI> YEAR,
 * <LI> MONTH,
 * <LI> DAY_OF_MONTH
 * </OL>
 * @see java.util.Calendar
 * @return int
*/
    public int get(int field)
      {
       return calendar.get(field);
      }

/**
 * returns the calendar field as stored
 * @see java.util.Calendar
 * @return java.util.Calendar, or null if stored date value is empty
*/
    public Calendar getCalendar()
      {
       if (value.length() ==0) return null;
       return calendar;
      }

/**
 * sets the value stored using an input string in the format YYYYMMDD where
 * <UL> 
 * <LI> YY is year
 * <LI> MM is month, 
 * <LI> DD is day of month
 *</UL>
 * @param   String field in format YYYYMMDD
*/
    public void set(String inDate)
      {
         if (inDate.length() == 0) 
          {
           value = "";
           return;
          }
         if (inDate.length() == 6) 
           {
             if (Integer.parseInt(inDate.substring(0,2)) < 50)
             {
               System.err.println("Date not Y2K compliant '20' will be prepended");
               calendar.set(Calendar.YEAR,  2000 + Integer.parseInt(inDate.substring(0,2)));
             }  
             else 
             {
               System.err.println("Date not Y2K compliant '19' will be prepended");
               calendar.set(Calendar.YEAR,  1900 + Integer.parseInt(inDate.substring(0,2)));
             }  
             calendar.set(Calendar.MONTH,  Integer.parseInt(inDate.substring(2,4))-1);
             calendar.set(Calendar.DAY_OF_MONTH,  Integer.parseInt(inDate.substring(4,6)));
           }
         else 
         if (inDate.length() == 8) 
           {
             calendar.set(Calendar.YEAR,  Integer.parseInt(inDate.substring(0,4)));
             calendar.set(Calendar.MONTH,  Integer.parseInt(inDate.substring(4,6))-1);
             calendar.set(Calendar.DAY_OF_MONTH,  Integer.parseInt(inDate.substring(6,8)));
           }

          value = new String(inDate);
           
      }

/**
 * sets a specific part of the date field
 * @param int field identifier
 * <BR>use the java.util.Calendar fields:
 * <OL>
 * <LI> YEAR,
 * <LI> MONTH,
 * <LI> DAY_OF_MONTH
 * </OL>
 * @param int value
 * @see java.util.Calendar
*/
    public void set(int field, int value)
      {
        calendar.set(field, value);
      }

;
 /**
  * formats text of data element
  * <br> Short Description of DataElement is defined in the class
  * <br> value is the current value set in the object
  * @param int formatType
  */

 public String getFormattedText(int formatType)
    {

    SimpleDateFormat sdf = new SimpleDateFormat("MMM dd, yyyy");
    StringBuffer sb = new StringBuffer();
    switch (formatType)
     {
      case TransactionSet.XML_FORMAT:
        sb.append("<" +getXMLTag() + ">" + value + "</" + getXMLTag() + ">\n");
        break;
      case TransactionSet.X12_FORMAT:
      case TransactionSet.EDIFACT_FORMAT:
           sb.append(value);
           break;
      default:
        sb.append("\t" + getDescription() + ": " + sdf.format(calendar.getTime()) + "\n");
     }   
    return new String(sb);
    }


 }

