package com.americancoders.edi.EDIFact;

/**
 *OBOE - Open Business Objects for EDI
 *<P>java access to EDI 
 *<p>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<p>All rights reserved
 *<p>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/


import java.util.*;
import java.io.*;
import java.net.*;

import com.americancoders.edi.*;
import com.americancoders.edi.EDIFact.*;


/**
 * class to parse input string for all defined ediObjects Transaction Sets
 * <br> 
 * <br> 
 *
*/

public class Parser 
 {
       URL url=null;

       TransactionSet parsedTransactionSet; 
       
       Segment UNA_Service_String = null,
               UNB_Interchange_Header = null,
               UNG_Functional_Group_Header = null,
               UNH_Message_Header = null,
               UNT_Message_Trailer = null,
               UNE_Functional_Group_Trailer = null,
               UNZ_Interchange_Trailer = null;
               
/**
 * create a transaction set from input string 
 * @param String the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(String inString)
      throws Exception
    {
        ParseString(inString);
    }

/**
 * builds  a transaction set by reading a file
 * @param File the filename containing the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(File inFile)
      throws Exception
    {
         byte b[] = new byte[(int) inFile.length()];
         FileInputStream fis = new FileInputStream(inFile);
         fis.read(b);
         ParseString(new String(b));
    }
/**
 * builds  a transaction set by reading a file from a url address
 * @param URL the URL containing the edi document 
 * @param String input text
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(URL inURL, String text)
      throws Exception
    {
         url = inURL;            
         ParseString(text);
    }


/**
 * builds  a transaction set from input string 
 * @param inString the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public void ParseString(String inString)
      throws Exception
    {
       int segCount; 
       int deCount;

       int posStart;
       boolean startsWithUNA = false;
       
       posStart = inString.indexOf("UNA");
       if (posStart > -1)
         startsWithUNA = true;
       else {
          posStart = inString.indexOf("UNB");
            if (posStart < 0) 
              throw new OBOEException("UNB (and UNA) segment missing");
       }    
       int posStop = inString.indexOf("UNZ");
       if (posStop < 0)
          throw new OBOEException("UNZ segment missing");

       Tokenizer et = new Tokenizer(inString.substring(posStart), "EDIFACT");  // , posStop-1));
       segCount = et.countSegments();
       et.getNextSegment();

	   //UNA
	   if (startsWithUNA){
	  //  UNA_Service_String = Service_String.
  	    et.getNextSegment();
	   }
	   //UNB
       String findID =  et.getNextDataElement();
       if (findID.compareTo(Interchange_Header.id) != 0)
          throw new OBOEException("Segment ID " + findID + " found, was expecting " + Interchange_Header.id + ".");
	   UNB_Interchange_Header = Interchange_Header.getInstance();
	   UNB_Interchange_Header.parse(et);
	   
       findID =  et.getCurrentDataElement();
       if (findID.compareTo(Functional_Group_Header.id) == 0){
          UNG_Functional_Group_Header = Functional_Group_Header.getInstance();
          UNG_Functional_Group_Header.parse(et);
          findID =  et.getCurrentDataElement();
         }
	   //UNH
       if (findID.compareTo(Message_Header.id) != 0)
          throw new OBOEException("Segment ID " + findID + " found, was expecting " + Message_Header.id + ".");
       UNH_Message_Header = Message_Header.getInstance();
       UNH_Message_Header.parse(et);
       
       findID = UNH_Message_Header.getCompositeDE("S009").getDataElement("0065").get();

       if (url != null)
          parsedTransactionSet = TransactionSetFactory.buildTransactionSet(findID, url);
       else   
          parsedTransactionSet = TransactionSetFactory.buildTransactionSet(findID);
       
       parsedTransactionSet.parse(et);

        // when we come out next segment is gotten and next data element is pointed to

       et.resetSegment();
       findID =  et.getCurrentDataElement();
       if (findID.compareTo(Functional_Group_Trailer.id) == 0)
       {
          UNE_Functional_Group_Trailer = Functional_Group_Trailer.getInstance();
          UNE_Functional_Group_Trailer.parse(et);
          findID =  et.getCurrentDataElement();
       }
       if (findID.compareTo(Interchange_Trailer.id) != 0)
         {
           throw new OBOEException("More segments to process starting at position " + 
             et.getSegmentPos()+ ". Problem " + et.getCurrentDataElement() + " is not defined.");
         }
      else {
         UNZ_Interchange_Trailer = Interchange_Trailer.getInstance();
         UNZ_Interchange_Trailer.parse(et);
      }

    }

/**
 * returns the transaction set that was parsed 
 * @return TransactionSet - the transaction set parsed when object was built.
*/
  public TransactionSet getTransactionSet()
    {
        return parsedTransactionSet;
    }

  
 }
