package com.americancoders.edi;


import java.util.*;
import java.io.*;
import java.net.*;

import com.americancoders.edi.*;
import com.americancoders.edi.x12.*;
import org.w3c.dom.*;

import org.xml.sax.*;

/**
 * class builds edi object by parsing input string in xml edi format
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
 *
*/

public class EDIXMLParser 
 {
       protected URL url=null;
       protected TransactionSet parsedTransactionSet; 
       
/**
 * create a transaction set from input string 
 * @param inString the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public EDIXMLParser(String inString)
      throws Exception
    {
        parseString(inString);
    }

/**
 * builds  a transaction set by reading a file
 * @param inFile the filename containing the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public EDIXMLParser(File inFile)
      throws Exception
    {
         parseFile(inFile);
    }
/**
 * builds  a transaction set by reading a file <i>NOT USED</i>
 * @param URL
 * @param String
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public EDIXMLParser(URL inURL, String text)
      throws Exception
    {
         url = inURL;            
         parseString(text);
    }

/** 
 * static main method to test xmlparser class
 *
 */
public static void main(String args[])
  {
    try {
       EDIXMLParser xmlp = new EDIXMLParser(new File(args[0]));
       System.out.println(xmlp.getTransactionSet().getFormattedText(1));
    }
    catch (Exception e1)
    {
        e1.printStackTrace();
    }    
  }
  
  
/**
 * builds  a transaction set from input string 
 * @param String the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public void parseString(String inString)
      throws Exception
      {
        ByteArrayInputStream bais = new ByteArrayInputStream(inString.getBytes());
        parseStream(bais);
      }  
      
/**
 * builds  a transaction set from input file 
 * @param File the edi document file name
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public void parseFile(File inString)
      throws Exception
    {
        FileInputStream fis = new FileInputStream(inString);
        parseStream(fis);
      }  
      
/**
 * builds  a transaction set from input stream
 * @param InputStream the edi document in a stream
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public void parseStream(InputStream inStream)
      throws Exception
    {
       int segCount; 
        try {
            org.apache.xerces.parsers.DOMParser parser = 
                (org.apache.xerces.parsers.DOMParser)
                    Class.forName("org.apache.xerces.parsers.DOMParser").newInstance();       
            parser.parse(new InputSource(inStream));
            Document document = parser.getDocument();
  			
           if (document.getNodeType() != Node.DOCUMENT_NODE) 
    		   throw new OBOEException("unknown node type found, should be DOCUMENT_NODE");
		
   		   if (parsedTransactionSet != null)
		      throw new OBOEException("document type already found");
			
           NodeList nl = document.getChildNodes();
                      
           String tsName = nl.item(0).getNodeName();
           
				
           String OBOEPropertiesFileName = "OBOE.properties";
           File f3, f2, f1 = new File(OBOEPropertiesFileName);
           if (f1.exists() == false)
             {
               OBOEPropertiesFileName = System.getProperty("user.home") + "/OBOE.properties";
               f2 = new File(OBOEPropertiesFileName);
               if (f2.exists() == false)
                 {
                   OBOEPropertiesFileName = System.getProperty("java.home") + "/OBOE.properties";
                   f3 = new File(OBOEPropertiesFileName);
                   if (f3.exists() == false)
                     {
                       System.err.println("Searched for OBOE.properties as " + f1.getAbsolutePath());
                       System.err.println("Searched for OBOE.properties as " + f2.getAbsolutePath());
                       System.err.println("Searched for OBOE.properties as " + f3.getAbsolutePath());
                       throw new OBOEException("OBOE.properties file not found");
                     }
                 } 
             }    
           Properties properties = new Properties();
           FileInputStream fis = new FileInputStream(OBOEPropertiesFileName);
           properties.load(fis);
           String tsID = properties.getProperty(tsName);
           
		   parsedTransactionSet = TransactionSetFactory.buildTransactionSet(tsID);
		   parsedTransactionSet.parse(nl.item(0));
		  
          }
		  
        catch (Exception e) {
            throw(e);
        }

    } 

/**
 * returns the transaction set that was parsed 
 * @return TransactionSet - the transaction set parsed when object was built.
*/
  public TransactionSet getTransactionSet()
    {
        return parsedTransactionSet;
    }

 }
