package com.americancoders.edi;


import java.util.*;

/**
 * class for all Data Elements defined as character
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class IDDE extends DataElement
 {
     protected IDList idList;

/**
 * constructs the IDDE type
 * @param String id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 *
*/
public IDDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
{
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        idList = null;
}

/**
 * constructs the IDDE type
 * @param String id
 * @param int sequence 
 * @param char required
 * @param String description
 * @param int minimum allowed length
 * @param int maximum allowed length
 *
*/
public IDDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
{
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        idList = null;
}

/**
 * constructs the ID DE type and sets its value
 * @param String id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 *
*/
public IDDE(String inValue, String inID, int inSequence, String inDesc,  char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
{
        super(inValue, inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        idList = null;
}
/**
 * constructs the IDDE type
 * @param String id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 * @param IDList inIDList
 *
*/
public IDDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag, IDList inIDList)
{
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        idList = inIDList;
}

/**
 * constructs the IDDE type
 * @param String id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param IDList inIDList
 *
*/
public IDDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength,  IDList inIDList)
{
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        idList = inIDList;
}

/**
 * constructs the ID DE type and sets its value
 * @param String id
 * @param int sequence 
 * @param String description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 * @param IDList inIDList
 *
*/
public IDDE(String inValue, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag,  IDList inIDList)
{
        super(inValue, inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        idList = inIDList;
}


  /**
  * sets the validating list with valid codes and description array
  * <br> used by Segment class, because codes are dependent on the Segment
  * @param String  valid code
  * @param String  valid code description or meaning
  */
  public void addCode(String inValidCode, String inCodeMeaning)
     {
        idList.add(inValidCode, inCodeMeaning);
     }

  /**
  * tests input string for validity
  * @param String code to test for
  * @return boolean - true if valid, false otherwise
  *
  */
   public boolean isCodeValid(String testCode)
     {
       if (idList == null) // then any value is valid
         return true;
       return idList.isCodeValid(testCode);
     }

   /**
   * returns a description for the current value
   * @return String -description of the code
   */
   public String describe() 
   {

       return describe(value);
   }
   /**
   * returns a description for a code
   * @param String code to search for
   * @return String -description of the code
   */
   public String describe(String testCode) 
   {

       if (idList == null)  // then any value is valid
	      return testCode;
	   return idList.describe(testCode);
   }

   /**
   * returns a code for a desciption
   * @param String description to search for
   * @return String code of the description
   */
   public String getCode(String testDescription) 
   {

       if (idList == null)  // then any value is valid
	    return testDescription;
       return idList.getCode(testDescription);
   }

 /**
  * formats text of data element                     
  * <br> Description of DataElement is defined in the class
  * <br> value is the current value set in the object
  * @param int formatType
  * <ul>
  * <li> formatType == 1 produces XML output
  * <li> default produces "description: value"
  * </ul> 
  */

 public String getFormattedText(int formatType)
    {
      StringBuffer sb = new StringBuffer();
      switch (formatType)
        {
         case TransactionSet.XML_FORMAT:
           if (formatType == TransactionSet.XML_FORMAT)
                sb.append("<" +getXMLTag() + ">" + value + "</" + getXMLTag() + ">\n");
            break;
         case TransactionSet.X12_FORMAT:
             return value;
         case TransactionSet.EDIFACT_FORMAT:
              return value;
         default:
             sb.append("\t" + getDescription() + ": " + describe() + "\n");
             break;
        }     
       return new String(sb);
    }

 }
