package com.americancoders.edi;

import org.xml.sax.*;
import java.util.Vector;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

public class IDList implements Externalizable
{
    protected String      name;
    protected Vector	  codes, values;
   /**
    * Construct an id list object with no values
    */
   public IDList(){}
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
   public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
     {
    name = in.readUTF();
    codes = (Vector) in.readObject();
    values = (Vector) in.readObject();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
   public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(name);
    out.writeObject(codes);
    out.writeObject(values);
   }
         
    /**
     * return name of id list
     * 
     * @return String
     */
    public String getName() {return name;}
    
    
    /**
     * Add code and description to the vectors
     * 
     * @param inCode
     * @param inDescribe
     */
    public void add(String inCode, String inDescribe)
    {
        codes.add(inCode);
        values.add(inDescribe);
    }

    /**
     * tests if the passed code is in the code vector
     * 
     * @param inCode
     * @return boolean
     */
    public boolean isCodeValid(String inCode)
    {
        String test;
        for (int i = 0; i < codes.size(); i++)
        {
            test = (String) codes.elementAt(i);
            if (test.compareTo(inCode) == 0) return true;
        }
        return false;
    }
    
    /**
     * returns the descriptive value of the code,  if it is not found then the code is returned
     * 
     * @param inCode
     * @return String
     */
    public String describe(String inCode)
    {
        String test;
        for (int i = 0; i < codes.size(); i++)
        {
            test = (String) codes.elementAt(i);
            if (test.compareTo(inCode) == 0) return (String) values.elementAt(i);
        }
        return inCode;
    }
    /**
     * returns the code value for a descriptive value.  if it is not found the value is returned
     * 
     * @param inValue
     * @return String
     */
    
    public String getCode(String inValue)
    {
        String test;
        for (int i = 0; i < values.size(); i++)
        {
            test = (String) values.elementAt(i);
            if (test.compareTo(inValue) == 0) return (String) codes.elementAt(i);
        }
        return inValue;
    }

    /**
     * returns a code at a specific position in vector
     * 
     * @param pos
     * @return String
     */
    public String getCode(int pos) { return (String) codes.elementAt(pos); }
    /**
     * returns the code vector
     * 
     * @return Vector
     */
    
    public Vector getValues() { return values;}
    
}
