package com.americancoders.edi;

import java.util.*;
import java.text.*;
import java.io.*;
import javax.mail.*;
import javax.mail.Flags.*;
import javax.mail.internet.*;
import javax.activation.*;
import java.security.*;
import java.security.interfaces.*;
import javax.crypto.*;



import com.americancoders.util.*;


/** 
* Class for shipping EDI Documents via Internet mail
* <br>Class contains a main method to allow it to invoked as an application.
* <br>format: java com.americancoders.edi.IncomingEDIMail hostid userid password mailServerProtocol
* <br>where hostid is the name of your incoming mail server
* <br>      userid mailid to get mail
* <br>      password
* <br>      mailServerProtocol - imap | pop3
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

	
public class IncomingEDIMail {

    static String mbox = "INBOX";
    static String url = null;
    static int port = -1;
    static String host, user, password, protocol;
    static boolean debug;
    static String ksFilename;
    static String ksPassword;

/** 
*  main logic for testing 
* <br>format: java com.americancoders.edi.IncomingEDIMail hostid userid password mailServerProtocol
* <br>where hostid is the name of your incoming mail server
* <br>      userid mailid to get mail
* <br>      password
* <br>      mailServerProtocol - imap | pop3
* <br> output sent to System.out
*/

 
 
    public static void main(String argv[]) {
        if (argv.length == 4)
            new IncomingEDIMail(argv[0], argv[1], argv[2], argv[3], true).getEDIDocument();
        else 
            System.out.println("format: java getEDIDocument host, user, password, protocol");
        
    }
	
/** 
 *  instatiate class 
 *  @param host String identifying internet host id
 *  @param user String identifying internet mail userid 
 *  @param password String identifying password used by host to verify userid
 *  @param protocol String mail protocol - imap or pop3
 *
 *
 */
 
public IncomingEDIMail(String inHost, String inUser, String inPassword, String inProtocol)
 {
    
    host = inHost;
    user = inUser;
    password = inPassword;
    protocol = inProtocol;
    debug = false;
 }   
	
	
/** 
 *  instatiate class 
 *  @param host String identifying internet host id
 *  @param user String identifying internet mail userid 
 *  @param password String identifying password used by host to verify userid
 *  @param protocol String mail protocol - imap or pop3
 *  @param debug boolean if true mail logic sent to System.out stream
 *
 *
 */
 
public IncomingEDIMail(String inHost, String inUser, String inPassword, String inProtocol, boolean inDebug)
 {
    
    host = inHost;
    user = inUser;
    password = inPassword;
    protocol = inProtocol;
    debug = inDebug;
 }   
	
	
public IncomingEDIMail(String inHost, String inUser, String inPassword, String inProtocol, String inKeyStoreFile, String inKSPassword)
 {
    
    host = inHost;
    user = inUser;
    password = inPassword;
    protocol = inProtocol;
    debug = false;
    ksFilename = inKeyStoreFile;
    ksPassword =  inKSPassword;
 }   
	
	
/** 
 *  instatiate class 
 *  @param host String identifying internet host id
 *  @param user String identifying internet mail userid 
 *  @param password String identifying password used by host to verify userid
 *  @param protocol String mail protocol - imap or pop3
 *  @param debug boolean if true mail logic sent to System.out stream
 *
 *
 */
 
public IncomingEDIMail(String inHost, String inUser, String inPassword, String inProtocol, String inKeyStoreFile,  String  inKSPassword, boolean inDebug)
 {
    
    host = inHost;
    user = inUser;
    password = inPassword;
    protocol = inProtocol;
    ksFilename = inKeyStoreFile;
    ksPassword =  inKSPassword;
    debug = inDebug;
 }   
	
	
/** method to dump mail to System.out */	
	
public void getEDIDocument()
    {
        

    try {
    Properties props = System.getProperties();

	// Get a Session object
	Session session = Session.getDefaultInstance(props, null);
	session.setDebug(debug);

	// Get a Store object
	Store store = null;
	if (url != null) {
	URLName urln = new URLName(url);
	store = session.getStore(urln);
	store.connect();
	    } else {
	if (protocol != null)		
		    store = session.getStore(protocol);
	else
	       store = session.getStore();

	// Connect
	if (host != null || user != null || password != null)
		    store.connect(host, port, user, password);
    else
		    store.connect();
	    }
	    
   // Open the Folder
    Folder folder = store.getDefaultFolder();
    if (folder == null) {
	        System.out.println("No default folder");
	        System.exit(1);
	    }

    folder = folder.getFolder(mbox);
    if (folder == null) {
	        System.out.println("Invalid folder");
	        System.exit(1);
	    }

    folder.open(Folder.READ_WRITE);
    int totalMessages = folder.getMessageCount();

    if (totalMessages == 0) {
		System.out.println("Empty folder");
		folder.close(false);
		store.close();
		System.exit(1);
	    }

	   
		// Attributes & Flags for all messages ..
		Message[] msgs = folder.getMessages();

		// Use a suitable FetchProfile
		FetchProfile fp = new FetchProfile();
		fp.add(FetchProfile.Item.ENVELOPE);
		fp.add(FetchProfile.Item.FLAGS);
		fp.add("X-Mailer");
		folder.fetch(msgs, fp);

		for (int i = 0; i < msgs.length; i++) {
		    dumpPart(msgs[i]);
		}
	    folder.close(false);
	    store.close();
	} catch (Exception ex) {
	    System.out.println("Oops, got exception! " + ex.getMessage());
	    ex.printStackTrace();
	}
	System.exit(1);
    }


/**
 * used by EDIMailCall class to store mail in a vector
 *  @param emailVector stored mail information
 */
	
public static void vectorizeEDIDocument(Vector emailVector)
	    {
	    boolean debug = true;    
	
	    try {
	    Properties props = System.getProperties();
	
		// Get a Session object
		Session session = Session.getDefaultInstance(props, null);
		session.setDebug(debug);
	
		// Get a Store object
		Store store = null;
		if (url != null) {
		URLName urln = new URLName(url);
		store = session.getStore(urln);
		store.connect();
		    } else {
		if (protocol != null)		
			    store = session.getStore(protocol);
		else
		       store = session.getStore();
	
		// Connect
		if (host != null || user != null || password != null)
			    store.connect(host, user, password);
	    else
			    store.connect();
		    }
		    
	   // Open the Folder
	    Folder folder = store.getDefaultFolder();
	    if (folder == null) {
		        System.out.println("No default folder");
		        System.exit(1);
		    }
	
	    folder = folder.getFolder(mbox);
	    if (folder == null) {
		        System.out.println("Invalid folder");
		        System.exit(1);
		    }
		    
		    
	
	    folder.open(Folder.READ_WRITE);
	    
	    
	    int totalMessages = folder.getMessageCount();
	
	    if (totalMessages == 0) {
			folder.close(false);
			store.close();
			return;
		    }
	
		   
		// Attributes & Flags for all messages ..
    	Message[] msgs = folder.getMessages();
			
		// Use a suitable FetchProfile
			FetchProfile fp = new FetchProfile();
			fp.add(FetchProfile.Item.ENVELOPE);
			fp.add(FetchProfile.Item.FLAGS);
			fp.add("X-Mailer");
			folder.fetch(msgs, fp);
	
			for (int i = 0; i < msgs.length; i++) {
			    vectorizeMail(emailVector, msgs[i]);
			}
		    folder.close(false);
		    store.close();
		} catch (Exception ex) {
		    System.out.println("Oops, got exception! " + ex.getMessage());
		    ex.printStackTrace();
		}
   }
	    

	    
/**
 * used by EDIMailCall class to remove email from server
 *  @param deletedVector list of mail to be deleted indentified by position on server
 */
	
public static void deleteEDIMail(Vector deletedVector)
	    {
	
	    try {
	    Properties props = System.getProperties();
	
		// Get a Session object
		Session session = Session.getDefaultInstance(props, null);
		session.setDebug(debug);
	
		// Get a Store object
		Store store = null;
		if (url != null) {
		URLName urln = new URLName(url);
		store = session.getStore(urln);
		store.connect();
		    } else {
		if (protocol != null)		
			    store = session.getStore(protocol);
		else
		       store = session.getStore();
	
		// Connect
		if (host != null || user != null || password != null)
			    store.connect(host, port, user, password);
	    else
			    store.connect();
		    }
		    
	   // Open the Folder
	    Folder folder = store.getDefaultFolder();
	    if (folder == null) {
		        System.out.println("No default folder");
		        System.exit(1);
		    }
	
	    folder = folder.getFolder(mbox);
	    if (folder == null) {
		        System.out.println("Invalid folder");
		        System.exit(1);
		    }
		    
		    
	
	    folder.open(Folder.READ_WRITE);
	    
	    
	    int totalMessages = folder.getMessageCount();
	
	    if (totalMessages == 0) {
			folder.close(false);
			store.close();
			return;
		    }
	
		   
		// Attributes & Flags for all messages ..
    	Message[] msgs = folder.getMessages();
			
        int i, m;
        for (i = 0; i < deletedVector.size(); i++)
            {
                m = ((Integer) deletedVector.elementAt(i)).intValue();
                msgs[m].setFlag(Flags.Flag.DELETED, true);
            }
            
	        
		folder.close(true);
		store.close();
		} catch (Exception ex) {
		    System.out.println("Oops, got exception! " + ex.getMessage());
		    ex.printStackTrace();
		}
//		System.exit(1);
	    }

/**
 * used by vectorizeEDIDocument to parse individual mail documents
 * <br>vector parts list
 * <ol>
 * <li> email message vector see vectorEnvelope
 * </ol>
 *  @param emailVector stored mail information
 *  @param p Part - mime part in document
 */
	
public static void vectorizeMail(Vector emailVector, Part p)
       throws Exception
	  {
        
    	if (p instanceof Message)
    	    emailVector.addElement(vectorizeEnvelope((Message)p, emailVector.size()));

    	Object o = p.getContent();
    	if (o instanceof String) {
		    return;
    	} else if (o instanceof Multipart) {
    	    Multipart mp = (Multipart)o;
    	    int count = mp.getCount();
    	    for (int i = 0; i < count; i++) {
    		    vectorizeMail(emailVector, mp.getBodyPart(i));
    		    return; 
    		}  
    	} else if (o instanceof Message) {
    	    vectorizeMail(emailVector, (Part)o);
    	} else if (o instanceof InputStream) {
    	    return;
    	}
	  }
	  
	  
/**
 * from javasoft sample program to display mail
 *  @param p Part - mime part in document
 */
	
public static void dumpPart(Part p) throws Exception {
	if (p instanceof Message)
	    dumpEnvelope((Message)p);

    System.out.println(p.getContentType());

	Object o = p.getContent();
	if (o instanceof String) {
	    System.out.println((String)o);
	} else if (o instanceof Multipart) {
	    Multipart mp = (Multipart)o;
	    int count = mp.getCount();
	    for (int i = 0; i < count; i++)
		dumpPart(mp.getBodyPart(i));
	} else if (o instanceof Message) {
	    dumpPart((Part)o);
	} else if (o instanceof InputStream) {
	    InputStream is = (InputStream)o;
	    int c;
	    while ((c = is.read()) != -1)
		System.out.write(c);
	}
    }

/**
 * from javasoft sample program to display mail
 *  @param p Part - mime part in document
 */
	
public static String getPartText(Part p, String from) throws Exception {
    
	if (p instanceof Message)
	    ; // nop for now

	Object o = p.getContent();

    if (p.isMimeType("application/pkcs7-mime"))
      {
	    InputStream is = (InputStream)o;
	    ByteArrayDataSource  bads = new ByteArrayDataSource(is, "application/pkcs7-mime");
        DataHandler dh = new DataHandler(bads);
        bads.getData();
	    byte c[] = bads.getData();
	try 
	  {
	    byte obt[];
	    KeyStore ks = KeyStore.getInstance("JCEKS");   
	    ks.load(new FileInputStream(ksFilename), ksPassword.toCharArray());
        Cipher cipher = Cipher.getInstance("RSA");        //get their public key and encrypt the message with it
        cipher.init(Cipher.DECRYPT_MODE, ks.getKey(from, ksPassword.toCharArray()));
        obt = cipher.doFinal(OutgoingEDIMail.decode(c));
        System.out.println(new String(obt));
        ByteArrayInputStream bais = new ByteArrayInputStream(obt);
        MimeBodyPart mbp = new MimeBodyPart(bais);
        MimePartDataSource mpds = new MimePartDataSource(mbp);
        System.out.println(mpds.getContentType());
        return getPartText((Part) mbp, from);
	  }
    catch(Exception e1)
      {
        e1.printStackTrace();    
        return "";
      }

      }
       
    else 
    {
	if (o instanceof String) {
	    return (String)o;
	    
	} else if (o instanceof Multipart) {
	    Multipart mp = (Multipart)o;
	    int count = mp.getCount();
	    // for (int i = 0; i < count; i++)
		return getPartText(mp.getBodyPart(0), from);
	} else if (o instanceof Message) {
	    return getPartText((Part)o, from);
	} else if (o instanceof InputStream) {
	    InputStream is = (InputStream)o;
	    byte c[] = new byte[1];
	    StringBuffer sb = new StringBuffer();
	    while ((c[0] = (byte) is.read()) != -1)
		   sb.append(new String(c));
		return new String(sb);   
	}
	}
	return null;
    }

/**
 * used by vectorizeMail to parse mail information and store in secondary mail vector
 * <br>vector parts list
 * <ol>
 * <li> sent date
 * <li> address from
 * <li> subject
 * <li> delete flag
 * <li> content type
 * <li> msg number
 * <li> msg text
 * </ol>
 *  @param m - mime message
 *  @param msgNo integer used to store the message number in the mail vectory
 
 */
	
public static Vector vectorizeEnvelope(Message m, int msgNo) throws Exception {
	   Vector msgVector = new Vector(6);
	   
      SimpleDateFormat formatter = new SimpleDateFormat();
      
    	Date d = m.getSentDate();
	    if (d != null)  msgVector.addElement(formatter.format(d));
	    else msgVector.addElement("Unknown Date");
		
	   Address[] a;
	   a = m.getFrom();
	   if (a != null) msgVector.addElement(a[0].toString());
	   else  msgVector.addElement("No FROM Address");
	                
 	   msgVector.addElement(m.getSubject());
 	   
       boolean deleted = false;
       
       Flags.Flag[] sf = m.getFlags().getSystemFlags(); // get the system flags
	   for (int i = 0; i < sf.length; i++) {
          if (sf[i] == Flags.Flag.DELETED)
		     deleted = true;
		   }
	   
	   msgVector.addElement(new Boolean(deleted));
	   
	   msgVector.addElement(m.getContentType());
	   
       // MDN Message Disposition Notifications
	   String[] mdn = m.getHeader("Disposition-Notification-To");
	   if (mdn != null) {
	    {
	    if (debug)
	      System.out.println("Disposition-Notification-To" + mdn[0]);
	    }  
	    msgVector.addElement(mdn[0]);
	   }
	   else {
	    {
	    if (debug)
	      System.out.println("No MDN Disposition-Notification-To");
	    }  
	    msgVector.addElement(null);
	   }
	    
	   msgVector.addElement(new Integer(msgNo));
	   msgVector.addElement(getPartText(m, m.getFrom().toString()));			

	   return msgVector;
	   
	}
	
	  
/**
 * from javasoft sample program to display mail
 *  @param m Message the mime message
 */
	
public static void dumpEnvelope(Message m) throws Exception {
	Address[] a;
	// FROM 
	if ((a = m.getFrom()) != null) {
	    for (int j = 0; j < a.length; j++)
		System.out.println("FROM: " + a[j].toString());
	}

	// TO
	if ((a = m.getRecipients(Message.RecipientType.TO)) != null) {
	    for (int j = 0; j < a.length; j++)
		System.out.println("TO: " + a[j].toString());
	}

	// SUBJECT
	System.out.println("SUBJECT: " + m.getSubject());

	// DATE
	Date d = m.getSentDate();
	System.out.println("SendDate: " +
	    (d != null ? d.toString() : "UNKNOWN"));

	// SIZE
	System.out.println("Size: " + m.getSize());

	// FLAGS:
	Flags flags = m.getFlags();
	StringBuffer sb = new StringBuffer();
	Flags.Flag[] sf = flags.getSystemFlags(); // get the system flags

	boolean first = true;
	for (int i = 0; i < sf.length; i++) {
	    String s;
	    Flags.Flag f = sf[i];
	    if (f == Flags.Flag.ANSWERED)
		s = "\\Answered";
	    else if (f == Flags.Flag.DELETED)
		s = "\\Deleted";
	    else if (f == Flags.Flag.DRAFT)
		s = "\\Draft";
	    else if (f == Flags.Flag.FLAGGED)
		s = "\\Flagged";
	    else if (f == Flags.Flag.RECENT)
		s = "\\Recent";
	    else if (f == Flags.Flag.SEEN)
		s = "\\Seen";
	    else
		continue;	// skip it
	    if (first)
		first = false;
	    else
		sb.append(' ');
	    sb.append(s);
	}

	String[] uf = flags.getUserFlags(); // get the user flag strings
	for (int i = 0; i < uf.length; i++) {
	    if (first)
		first = false;
	    else
		sb.append(' ');
	    sb.append(uf[i]);
	}
	System.out.println("FLAGS = " + sb.toString());


    // Content type
    System.out.println("Content type: " + m.getContentType());
    
	// MDN Message Disposition Notifications
	String[] mdn = m.getHeader("Disposition-Notification-To");
	if (mdn != null)
	    System.out.println("Disposition-Notification-To" + mdn[0]);
	else
	    System.out.println("Disposition-Notification-To NOT available");
    }
}
