package com.americancoders.edi;

import java.util.*;
import java.text.*;

/**
 * class for Data Elements defined as Numeric (Nx - where x is an integer representing number of decimal places)
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class NumericDE extends DataElement
 {

   protected int fractionDigits;

/**
 * constructs a NumericDE object
 * @param String x12 id
 * @param int sequence within structure
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
*/
   public NumericDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      fractionDigits = 0;
    }

/**
 * constructs a NumericDE object
 * @param String x12 id
 * @param int sequence within structure
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public NumericDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
      fractionDigits = 0;
    }

/**
 * constructs a NumericDE object using an input string
 * @param inValue             String field in a numeric format
 * @param String x12 id
 * @param int sequence within structure
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
*/
   public NumericDE(String inNumber, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      fractionDigits = 0;
      if (inNumber.length() == 0)
        value = "0";
      else
        value = inNumber;
    }


/**
 * constructs a NumericDE object using an input string
 * @param inValue             String field in a numeric format
 * @param String x12 id
 * @param int sequence within structure
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public NumericDE(String inNumber, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
      fractionDigits = 0;
      if (inNumber.length() == 0)
        value = "0";
      else
        value = inNumber;
    }

/**
 * constructs a NumericDE object
 * @param String x12 id
 * @param int sequence within structure
 * @param String x12 type  used to detemine the number of digits
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
*/
   public NumericDE(String inID, int inSequence, String inType, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      // for EDIFact
      if (inType.length() == 1) fractionDigits = 0;
      else
      fractionDigits = Integer.parseInt(inType.substring(1));
    }

/**
 * constructs a NumericDE object
 * @param String x12 id
 * @param int sequence within structure
 * @param String x12 type  used to detemine the number of digits
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public NumericDE(String inID, int inSequence, String inType, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
      fractionDigits = Integer.parseInt(inType.substring(1));
    }

/**
 * constructs a NumericDE object
 * @param String value
 * @param String x12 id
 * @param int sequence within structure
 * @param String x12 type  used to detemine the number of digits
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
*/
   public NumericDE(String inNumber, String inID, int inSequence, String inType, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      fractionDigits = Integer.parseInt(inType.substring(1));
      if (inNumber.length() == 0)
        value = "0";
      else
        value = inNumber;
    }

/**
 * constructs a NumericDE object
 * @param String x12 value
 * @param String x12 id
 * @param int sequence within structure
 * @param String x12 type  used to detemine the number of digits
 * @param String description
 * @param char required 
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public NumericDE(String inNumber, String inID, int inSequence, String inType, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
      fractionDigits = Integer.parseInt(inType.substring(1));
      if (inNumber.length() == 0)
        value = "0";
      else
        value = inNumber;
    }

/** 
 * gets the number of decimal positions
 * @return int number of decimal positions
 * 
*/

 public int getDecimalPositions() { return fractionDigits; }

/**
 * sets  NumericDE object using an input string
 * <br> use class set method to set values with assumed decimal
 * @param inValue    String field in a numeric format with decimal
*/
    public void setFormatted(String inNumber)
    {
      int fraction = 0;
      int integer =  0;
      int decPos = inNumber.indexOf('.');
      int i;
      if (decPos < 0)
        {
          fraction = 0;
          integer = Integer.parseInt(inNumber);
        }
     else
       {
         String preFraction = inNumber.substring(decPos+1);

         if (preFraction.length() > fractionDigits)
              fraction = Integer.parseInt(preFraction.substring(0, fractionDigits));
         else if (preFraction.length() < fractionDigits)
           {
            for (i = preFraction.length(); i < fractionDigits; i++)
               fraction *= 10;
           }
         else
              fraction = Integer.parseInt(preFraction);

         integer = Integer.parseInt(inNumber.substring(0, decPos));
       }

     for (i = 0; i < fractionDigits; i++)
        integer *= 10;

     value = "" + (integer + fraction);    

    }


  /**
   * builds a formatted String of the object
   * @param int type
   * @return String
   */
  public String getFormattedText(int formatType)
    {
      int i;
      if (formatType == TransactionSet.X12_FORMAT |
            formatType == TransactionSet.EDIFACT_FORMAT)
            return value;
      String returnString;      
      
      if (value.length() > 0) {
         NumberFormat nf = NumberFormat.getNumberInstance();
         nf.setMinimumFractionDigits(fractionDigits);
         nf.setMaximumFractionDigits(fractionDigits);
         nf.setMinimumIntegerDigits(minLength);
         nf.setMaximumIntegerDigits(maxLength);
         double d;
         try{ d = Double.valueOf(value.trim()).doubleValue(); 
         for (i = 0; i < fractionDigits; i++)
             d /= 10;
         returnString = nf.format(d);
         }
         catch (java.lang.NumberFormatException e1) {returnString = value;}
         
      }
      else returnString = value;

      StringBuffer sb = new StringBuffer();
      if (formatType == TransactionSet.XML_FORMAT)
         sb.append("<"+getXMLTag() + ">" + returnString + "</" + getXMLTag() + ">\n");
      else
         sb.append("\t"+getDescription() + ": " + returnString + "\n");
      return new String(sb);

    }


 }

