package com.americancoders.edi;

import java.io.*;
import java.util.*;

import javax.mail.*;
import javax.activation.*;

import javax.mail.internet.*;

import java.security.*;
import java.security.interfaces.*;
import javax.crypto.*;

import com.americancoders.util.ByteArrayDataSource;

  
/**
* Class to send EDI Documents via Internet Mail
* <br>Class contains a main method to allow it to invoked as an application.
* <br>format: java com.americancoders.edi.OutgoingEDIMail hostid userid toWhom filename
* <br>where hostid is the name of your incoming mail server
* <br>      userid mailid to get mail
* <br>      toWhom userid to receive file
* <br>      filename file to be mailed, file will be wrapped around MIME type applicaiton/edi/x12
 *<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/
	
public class OutgoingEDIMail {
    
    String host, user, protocol = "SMTP";
    String subject = "EDI/X12 Document Transmission";
    String to;
    String cc;
    String bcc;
    KeyStore ks;   
    static java.security.Provider pv[];

 /*
  * method for testing purposes
  * <br>format: java com.americancoders.edi.OutgoingEDIMail hostid userid toWhom filename
  * <br>where hostid is the name of your incoming mail server
  * <br>      userid mailid to get mail
  * <br>      toWhom userid to receive file
  * <br>      filename file to be mailed, file will be wrapped around MIME type applicaiton/edi/x12
 */
 
 public static void main(String[] argv)
    {
        if (argv.length != 4  && argv.length != 6 && argv.length != 7)
           {
            System.out.println("format: java OutgoingEDIMail host userid toWhom filename [keystorefilename keystorepassword [signature]]");
            System.out.println("assumes SMTP protocol");
            System.exit(1);
           }
           
           
         System.out.println(System.getProperty("user.home"));
        String sb = null;
        byte b[] = null;
         int readi = 0;
         int reads = 0;
         FileInputStream fis = null;
        try {
         fis = new FileInputStream(argv[3]);
         b = new byte[(int) fis.available()];
          }
         catch (Exception e1) {
            System.out.println(e1.getMessage());
            e1.printStackTrace();
            System.exit(1);
         }
            
         try {
           while (true)
            {

             reads = fis.available();
             if (reads > 0)
                 fis.read(b, readi, reads);
             else break;
             readi += reads;
            }
           }   
         catch (IOException e1) {}
   
        sb = new String(b, 0 , readi);
        if (argv.length == 4) {
            OutgoingEDIMail oem = new OutgoingEDIMail(argv[0], argv[1]);
            oem.sendEDIDocument( sb, argv[2], true);
        }
        else
        if (argv.length == 6) {
            try {
            OutgoingEDIMail oem = new OutgoingEDIMail(argv[0], argv[1], argv[4], argv[5]);
            oem.sendEncryptedEDIDocument( sb, argv[2], true);
            }
            catch (Exception e1)
            {
                e1.printStackTrace();
                System.exit(0);
            }
        }   
        else if (argv.length == 7) {
            try {
            OutgoingEDIMail oem = new OutgoingEDIMail(argv[0], argv[1], argv[4], argv[5]);
            oem.sendSignedEDIDocument( sb, argv[2], true);
            }
            catch (Exception e1)
            {
                e1.printStackTrace();
                System.exit(0);
            }
        }   
    }

 /** 
  * construct with host, user protocol.  as of now the only protocol supported is SMTP, so there's
  * really no reason to use this instantiator.
  * @param String SMTP hostid
  * @param String register userid
  * @param String protocol - if someone provides another java.mail protocol, specify it here
  */
  
 public OutgoingEDIMail(String inHost, String inUser, String inProtocol)
      {
        host = inHost;
        user = inUser;
        if (inProtocol != null)
            protocol = inProtocol;
      }

 /** 
  * construct with host, user protocol.  as of now the only protocol supported is SMTP
  * @param String SMTP hostid
  * @param String register userid
  */
 public OutgoingEDIMail(String inHost, String inUser)
      {
        host = inHost;
        user = inUser;
      }

 /** 
  * construct with host, user protocol.  as of now the only protocol supported is SMTP
  * @param String SMTP hostid
  * @param String register userid
  * @param String key Store Filename
  * @param String key Store Password
  */
 public OutgoingEDIMail(String inHost, String inUser, String inKeyStoreFileName, String inKeyStorePassword)
   throws Exception
      {
        host = inHost;
        user = inUser;
        System.out.println(System.getProperty("java.home"));
        ks = KeyStore.getInstance("JCEKS");   
        ks.load(new FileInputStream(inKeyStoreFileName), inKeyStorePassword.toCharArray());
      }

 /** 
  * send a document, uses the other sendEDIDocument with debugging turned off
  * @param String message, the EDI Document
  * @param String toWhom, target audience
  * @param boolean sendMDN, attach message disposition notification MIME object
  */
 
 public void sendEDIDocument(String message, String toWhom, boolean sendMDN)
    {
        sendEDIDocument(message, toWhom, sendMDN, false);  
    }

 
 
/** 
  * send a document, optional debugging switch.
  * <br> hardcoded mailer id
  * <br> hardcoded subject line, see class variable subject
  * <br> hardcoded mime type applciation/edix12
  * @param String message, the EDI Document
  * @param String toWhom, target audience
  * @param boolean sendMDN, attach message disposition notification MIME object
  * @param boolean debug turn on java.mail debugger 
  */
 public void sendEDIDocument(String message, String toWhom, boolean sendMDN, boolean debug) {
    
    try {
	String url = null;
	String mailer = "EDI/X12/OBOE";
	String record = null;	// name of folder in which to record mail
	
	System.getProperties().put("mail.smtp.host", host);
	
    Session session = Session.getDefaultInstance(System.getProperties(), null);
	if (debug)
		session.setDebug(true);

	MimeMessage mMsg = new MimeMessage(session);
	mMsg.setFrom(new InternetAddress(user));
	if (toWhom != null) 
	   to = toWhom;
	mMsg.setRecipients(Message.RecipientType.TO,
					InternetAddress.parse(to, false));
	mMsg.setSubject(subject);
	mMsg.setHeader("X-Mailer", mailer);
	if (sendMDN == true)
	    mMsg.addHeader("Disposition-Notification-To", user);
	mMsg.setSentDate(new Date());

    mMsg.setDataHandler(new DataHandler(
	     	new ByteArrayDataSource(message, "application/edix12")));
	    
    Transport.send(mMsg);
    
	} 
	catch (Exception e1) {e1.printStackTrace();}

	
	}
	
	
/**    
  * send an encrypted document, optional debugging switch.
  * <br> hardcoded mailer id
  * <br> hardcoded subject line, see class variable subject
  * <br> hardcoded mime type applciation/edix12
  * @param String message, the EDI Document
  * @param String toWhom, target audience, keyStore key
  * @param boolean debug turn on java.mail debugger 
 */ 
 public void sendEncryptedEDIDocument(String message, String toWhom,  boolean debug) {
    
    try {
	String url = null;
	String mailer = "EDI/X12/OBOE";
	String record = null;	// name of folder in which to record mail
	
	System.getProperties().put("mail.smtp.host", host);
	
    Session session = Session.getDefaultInstance(System.getProperties(), null);
	if (debug)
		session.setDebug(true);

	MimeMessage mMsg = new MimeMessage(session);
	mMsg.setFrom(new InternetAddress(user));
	if (toWhom != null) 
	   to = toWhom;
	mMsg.setRecipients(Message.RecipientType.TO,
					InternetAddress.parse(to, false));
	mMsg.setSubject(subject);
	mMsg.setHeader("X-Mailer", mailer);
	mMsg.setSentDate(new Date());
	byte obt[] = null;
	try 
	  {
	      pv = Security.getProviders();
        Cipher cipher = Cipher.getInstance("DES");        //get their public key and encrypt the message with it
        ks.getCertificate(to).getPublicKey();
        cipher.init(Cipher.ENCRYPT_MODE, ks.getCertificate(to).getPublicKey());
        
        InternetHeaders ih = new InternetHeaders();
        ih.addHeader("Content-Type:",  "application/edix12");
        MimeBodyPart mbp = new MimeBodyPart(ih, message.getBytes());
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        mbp.writeTo(baos);
        obt = baos.toByteArray();
        obt = cipher.doFinal(obt);
       
      }
    catch(Exception e1)
      {
       // LogFile.log("OutgoingEDIMail", e1.getMessage());
        e1.printStackTrace();    
        return;
      }

    
    mMsg.setDataHandler(new DataHandler(
	     	new ByteArrayDataSource(encode(obt), "application/pkcs7-mime")));
	
    Transport.send(mMsg);
    
	} 
	catch (Exception e1) {e1.printStackTrace();}

	
	}
	

/**    
  * send an signed document, optional debugging switch.
  * <br> hardcoded mailer id
  * <br> hardcoded subject line, see class variable subject
  * <br> hardcoded mime type applciation/edix12
  * @param String message, the EDI Document
  * @param String toWhom, target audience, keyStore key
  * @param boolean debug turn on java.mail debugger 
 */ 
 public void sendSignedEDIDocument(String message, String toWhom,  boolean debug) {
    
    try {
	String url = null;
	String mailer = "EDI/X12/OBOE";
	String record = null;	// name of folder in which to record mail
	
	System.getProperties().put("mail.smtp.host", host);
	
    Session session = Session.getDefaultInstance(System.getProperties(), null);
	if (debug)
		session.setDebug(true);

	MimeMessage mMsg = new MimeMessage(session);
	mMsg.setFrom(new InternetAddress(user));
	if (toWhom != null) 
	   to = toWhom;
	mMsg.setRecipients(Message.RecipientType.TO,
					InternetAddress.parse(to, false));
	mMsg.setSubject(subject);
	mMsg.setHeader("X-Mailer", mailer);
	mMsg.setSentDate(new Date());
	MimeMultipart mmp = new MimeMultipart("signed; \n micalg=SHA-1; \n protocol=\"application/pkcs7-signature\"");
	
	MimeBodyPart mbp1 = new MimeBodyPart();
	mbp1.setContent("application", "edix12");
	mbp1.setText(message);
	mmp.addBodyPart(mbp1);
	
	
	mMsg.setContent(mmp);
	byte obt[] = null;
    MimeBodyPart mbp2;
	try 
	  {
	   
        KeyGenerator kg = KeyGenerator.getInstance("HmacSHA1");
        SecretKey sk = kg.generateKey();
        Mac mac = Mac.getInstance("HmacSHA1");        //get their public key and encrypt the message with it
        mac.init(sk);
        
        InternetHeaders ih = new InternetHeaders();
        ih.addHeader("Content-Type",  "application/pkcs7-signature; name=smime.p7s");
        ih.addHeader("Content-Disposition",  "attachment; filename=smime.p7s");
        obt = mac.doFinal(message.getBytes());
        mbp2 = new MimeBodyPart(ih, encode(obt));
       
      }
    catch(Exception e1)
      {
       // LogFile.log("OutgoingEDIMail", e1.getMessage());
        e1.printStackTrace();    
        return;
      }
    
	mmp.addBodyPart(mbp2);
	
    Transport.send(mMsg);
    
	} 
	catch (Exception e1) {e1.printStackTrace();}

	
	}
	

/** 
  * send an MIME Message Delivery Notification back to sender
  * <br> hardcoded mailer id
  * <br> hardcoded subject line, see class variable subject
  * <br> hardcoded response text
  * <br> hardcoded reported as automatic-action in MDN disposition
  * @param String toWhom, target audience
  * @param boolean lookedAt, was the message looked at
  */
    public void sendMDNResponse(String toWhom, boolean lookedAt)
     {
        
    try {
	String url = null;
	String mailer = "EDI/X12/OBOE";
	String record = null;	// name of folder in which to record mail
	String responseText = "This is a Return Receipt for the mail that you sent.";

	System.getProperties().put("mail.smtp.host", host);
	
    Session session = Session.getDefaultInstance(System.getProperties(), null);
	MimeMessage mMsg = new MimeMessage(session);
	if (toWhom != null) 
	   to = toWhom;
	mMsg.setFrom(new InternetAddress(user));
	mMsg.setRecipients(Message.RecipientType.TO,
					InternetAddress.parse(to, false));
	mMsg.setSubject("Return Receipt (displayed) - " + subject);
	mMsg.setHeader("X-Mailer", mailer);
	mMsg.setSentDate(new Date());
	
	MimeMultipart mmp = new MimeMultipart("report; report-type=disposition-notification");
	
	MimeBodyPart mbp1 = new MimeBodyPart();
	mbp1.setContent("text", "plain");
	mbp1.setText(responseText);
	mmp.addBodyPart(mbp1);
	
	MimeBodyPart mbp2 = new MimeBodyPart();
    StringBuffer notificationText = new StringBuffer("Reporting-UA: OBOE; OutgoingEDIMail class (using javax.mail)\n");
    notificationText.append("Final-Recipient: rfc822;" + user + "\n");
    notificationText.append("Disposition: automatic-action/MDN-sent-automatically;");
    if (lookedAt) 
        notificationText.append("displayed\n");
    else    
        notificationText.append("ignored\n");
    mbp2.setDataHandler(new DataHandler(
	     	new ByteArrayDataSource(new String(notificationText), "message/disposition-notification")));
	mmp.addBodyPart(mbp2);
	
	mMsg.setContent(mmp);
	
    Transport.send(mMsg);
    
	} 
	catch (Exception e1) {e1.printStackTrace();}

     }
   

static public byte[] encode(byte[] data)
{
    int i, index;
    byte[] out = new byte[((data.length + 2) / 3) * 4];

    for (i=0, index=0; i<data.length; i+=3, index+=4) {
        boolean quad = false;
        boolean trip = false;

        int val = (0xFF & (int) data[i]);
        val <<= 8;
        if ((i+1) < data.length) {
            val |= (0xFF & (int) data[i+1]);
            trip = true;
        }
        val <<= 8;
        if ((i+2) < data.length) {
            val |= (0xFF & (int) data[i+2]);
            quad = true;
        }
        out[index+3] = letters[(quad? (val & 0x3F): 64)];
        val >>= 6;
        out[index+2] = letters[(trip? (val & 0x3F): 64)];
        val >>= 6;
        out[index+1] = letters[val & 0x3F];
        val >>= 6;
        out[index+0] = letters[val & 0x3F];
    }
    return out;
}

static public byte[] decode(byte[] data)
{

    int shift = 0;   // # of excess bits stored in accum
    int accum = 0;   // excess bits
    int index = 0;
    int ix = -1;
    int len = ((data.length + 3) / 4) * 3;
    if (data.length>0 && data[data.length-1] == '=') --len;
    if (data.length>1 && data[data.length-2] == '=') --len;
    byte[] out = new byte[len];

    for (ix=0; ix<data.length; ix++)
    {
        int value = codes[ data[ix] & 0xFF ];   // ignore high byte of char
        if ( value >= 0 ) {                     // skip over non-code
            accum <<= 6;            // bits shift up by 6 each time thru
            shift += 6;             // loop, with new bits being put in
            accum |= value;         // at the bottom.
            if ( shift >= 8 ) {     // whenever there are 8 or more shifted in,
                shift -= 8;         // write them out (from the top, leaving any
                out[index++] =      // excess at the bottom for next iteration.
                    (byte) ((accum >> shift) & 0xff);
    }   }   }
    if (index != out.length)
        throw new Error("miscalculated data length!");

    return out;
}

static private byte[] letters =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
        .getBytes();

static private byte[] codes = new byte[256];
static {
    int i;
    for (i=0; i<256; i++) codes[i] = -1;
    for (i = '0'; i <= '9'; i++) codes[i] = (byte)(52 + i - '0');
    for (i = 'a'; i <= 'z'; i++) codes[i] = (byte)(26 + i - 'a');
    for (i = 'A'; i <= 'Z'; i++) codes[i] = (byte)(     i - 'A');
    codes['+'] = 62;
    codes['/'] = 63;
}

}
