package com.americancoders.edi;

import java.util.*;
import java.text.*;

/**
 *  class for Data Elements defined as Real
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class RealDE extends DataElement
 {

   NumberFormat nf;

/**
 * constructs a RealDE object
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
*/
   public RealDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence,  inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      setNumberFormat();
    }

/**
 * constructs a RealDE object
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public RealDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence,  inDesc, inRequired, inMinLength, inMaxLength);
      setNumberFormat();
    }

/**
 * constructs a RealDE object using an input string
 * @param inValue String field in a numeric format
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
*/
   public RealDE(String inNumber, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
      setNumberFormat();
      value = inNumber;
    }

/**
 * constructs a RealDE object using an input string
 * @param inValue String field in a numeric format
 * @param String x12 id
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
   public RealDE(String inNumber, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
      super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
      setNumberFormat();
      value = inNumber;
    }

/**
 * sets the NumberFormat class variable
 */
  private void setNumberFormat()
  {
      nf = NumberFormat.getNumberInstance();
      nf.setMinimumFractionDigits(minLength);
      nf.setMaximumFractionDigits(minLength);
      nf.setMinimumIntegerDigits(1);
      nf.setMaximumIntegerDigits(maxLength);
  }

/** 
 * gets the number of decimal positions
 * @return int number of decimal positions
 * 
*/

 public int getDecimalPositions() { return minLength; }

/**
 * creates a RealDE object using an input string
 * @param inValue             String field in a numeric format
*/
  public String getFormattedText(int formatType)
    {
      int i;
      if (formatType == TransactionSet.X12_FORMAT |
            formatType == TransactionSet.EDIFACT_FORMAT)
            return value;
      NumberFormat nf = NumberFormat.getNumberInstance();
      StringBuffer sb = new StringBuffer();
      if (formatType == TransactionSet.XML_FORMAT)
         sb.append("<"+getXMLTag() + ">");
      else
         sb.append("\t"+getDescription() + ": ");
      try {
        sb.append(nf.format(Double.valueOf(value.trim()).doubleValue()));}
      catch (Exception e1) {sb.append(value);}
      if (formatType == 1)
         sb.append("</"+getXMLTag() + ">\n");
      else
         sb.append("\n");
      return new String(sb);

    }
 }
