package com.americancoders.edi;

import java.util.*;
import com.americancoders.edi.*;
import org.w3c.dom.*;


/**
 * class for Segments
 *OBOE - Open Business Objects for EDI
 *<P>An EDI and XML Translator Written In Java 
 *<p>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<p>All rights reserved
 *<p>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class Segment extends SegmentContainer implements IDataElementContainer, ICompositeDEContainer
 {
   
   protected String id = "";
   protected int sequence = 0;
   protected String description;
   protected Object deArray[];
   protected TemplateSegment myTemplate = null;
   protected int occurs = 0;
   protected char required;
   protected String xmlTag;

/** 
 * create a Segment based on its template
 * @param TemplateSegment predefined template
 *
 */
 
   public Segment(TemplateSegment inTemplateSegment)
     throws Exception
     {
        myTemplate = inTemplateSegment;
        
        setID(inTemplateSegment.getID());
        setSequence(inTemplateSegment.getSequence());
        setDescription(inTemplateSegment.getDescription());
        setOccurs(inTemplateSegment.getOccurs());
        setXMLTag(inTemplateSegment.getXMLTag());
        setRequired(inTemplateSegment.getRequired());
        templateSegmentVector = new Vector (inTemplateSegment.getTemplateSegmentSize());
        for (int i = 0; i < inTemplateSegment.getTemplateSegmentSize(); i++)
           templateSegmentVector.addElement(inTemplateSegment.getTemplateSegment(i));
       
        segmentArray = new Object[inTemplateSegment.getTemplateSegmentSize()];
        deArray = new Object[inTemplateSegment.getTemplateDESize()];
     }   

/** 
 * create a Segment 
 * @param String id
 * @param int sequence 
 * @param String description
 * @param int occurs
 * @param char required
 * @param String xmltag
 *
 */
 
   public Segment(String inId, int inSequence, String inShortDescription, int inOccurs, char inRequired, String inXMLTag)
     throws Exception
     {
        setID(inId);
        setSequence(inSequence);
        setDescription(inShortDescription);
        templateSegmentVector = new Vector();
        setOccurs(inOccurs);
        setXMLTag(inXMLTag);
        segmentArray = null;
        deArray = null;
     }   

/** 
 * create a Segment, defaults the xmltag to the field id value
 * @param String id
 * @param int sequence 
 * @param String description
 * @param int occurs
 8 @param char required
 *
 */
 
   public Segment(String inId, int inSequence, String inShortDescription, int inOccurs, char inRequired)
     throws Exception
     {
        setID(inId);
        setSequence(inSequence);
        setDescription(inShortDescription);
        templateSegmentVector = new Vector();
        setOccurs(inOccurs);
        setRequired(inRequired);
        setXMLTag(id.replace(' ','_'));
        segmentArray = null;
        deArray = null;
     }   
     
/**
 * returns the TemplateSegment used to build the Segment
 * @param TemplateSegment
 */
public TemplateSegment getTemplate(){ return myTemplate; }
/**
 * sets the segment id
 * @param String id
 */
public void setID(String inId){ id = inId; }

/**
 * gets the segment id
 * @return String id
 */
 
 
public String getID(){return id;}
 
/**
 * sets the occurs value
 * @param int occurance value
 */
 
 public void setOccurs(int inOccurs) {occurs = inOccurs;}
 
/**
 * returns the occurs value
 * @return int occurance value
 * 
 */
 
 public int getOccurs() {return occurs;}
 
/**
 * sets the required flag
 * @param char required flag
 */
 
 public void setRequired(char inRequired) {required = inRequired;}
 
/**
 * returns the required flag
 * @return char required
 * 
 */
 
 public char getRequired() {return required;}
 

/** 
 * sets the xmltag field
 * @param String xmltag
 */

 public void  setXMLTag(String inXMLTag) { xmlTag = inXMLTag; }
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
    
 public String getXMLTag() { return xmlTag; }
 
/**
 * sets the segment sequence
 * @param int sequence number
 */
public void setSequence(int inSequence){ sequence = inSequence; }

/**
 * gets the segment sequence
 * @return int sequence number
 */
public int getSequence(){ return sequence; }
    
 /**
  * this is here as a stub because of the segment container interface
  * @param String id
  * @return Segment
  */ 
  
public Segment createSegment(String ID)
   throws Exception
  {return myTemplate.createSegment(ID);}
   
   
/**
 * sets Description for the Segmemt
 * param String description
 *
*/
    public void setDescription(String inDesc) {description = inDesc;};

/**
 * returns the Short Description for the Segment
 * @return String
*/
    public String getDescription() {return description;};
	
	
/**
 * parses a EDI Document,  this maybe x12 dependent.
 * adds to datalement vector
 * and adds to secondary segment vector
 * @param ITokenizer input tokenized string
 * @param ITokenizer what is left
 *
*/

public ITokenizer parse(ITokenizer TransactionTokenizedString)
                throws Exception
   {
      int i;
      int elementsToParse;
      
      DataElement currentDE;
      CompositeDE currentComposite;
      Segment currentSegment;
      
      TransactionTokenizedString.getNextDataElement();
      elementsToParse = TransactionTokenizedString.countDataElements();
      if (id.compareTo("ISA") == 0) // don't count the separator again
         elementsToParse--;
         
      if (myTemplate == null || myTemplate.areYouAGroup() == false)
      for (i=0;
            i < elementsToParse && TransactionTokenizedString.hasMoreDataElements();
              i++)
        {
          if (i >= deArray.length || (deArray[i] == null))  
            {
             buildDE(i);
            }  
          if (isCompositeDE(i))
             {
                currentComposite = (CompositeDE) deArray[i];
                if (currentComposite.parse(TransactionTokenizedString) == false)
                  continue;
             }   
          else 
           {
              currentDE = (DataElement) deArray[i];
              currentDE.set(TransactionTokenizedString.getCurrentDataElement());
           }
          TransactionTokenizedString.getNextDataElement();
        }
        
      if (TransactionTokenizedString.getNextSegment() == null) return null;
      TransactionTokenizedString.getNextDataElement();
        
      if (this.id.compareTo("NTE") == 0)
        return TransactionTokenizedString;  // special notes don't have children 
        
      i = 0;
      TemplateSegment tseg;
      while (i < templateSegmentVector.size())
      {
        tseg = (TemplateSegment) templateSegmentVector.elementAt(i);
       if (TransactionTokenizedString.getCurrentDataElement().compareTo(tseg.getID()) == 0)
        {
          currentSegment = new Segment(tseg);
          addSegment(currentSegment);
          TransactionTokenizedString = currentSegment.parse(TransactionTokenizedString);
          if (currentSegment.getOccurs() == 1)
              i++; // don't look back
          //else let's try this one again
         }   
        else i++; // next
           
      }  
     return TransactionTokenizedString; 

   }
   
/**
 * parses a XML EDI Document
 * adds to datalement vector or compositeDE
 * and adds to secondary segment vector
 * @param Node input XML node
 *
*/

public void parse(Node node)
                throws Exception
   {
      int i=0, n, cn;
      int elementsToParse;
      
      NodeList nl = node.getChildNodes();
      NodeList cnl;
      DataElement currentDE;
      CompositeDE currentComposite;
      Segment currentSegment;
      Node currentNode;
      int lastDETested = -1;
      
      Node cnode;
      nodeLoop: for (n = 0; n < nl.getLength(); n++)
        {
          currentNode = nl.item(n);
          if (currentNode.getNodeType() != Node.ELEMENT_NODE)  continue nodeLoop;
          cnl = currentNode.getChildNodes();
          i = myTemplate.doYouUseThisXMLElement(currentNode.getNodeName(), lastDETested);
          if (i >= 0) 
            {
             if (i > lastDETested)
              {
                for (cn = lastDETested+1; cn <= i; cn++)
                  buildDE(cn); 
               }   
             lastDETested = i;
             if (isCompositeDE(i))
                   {
                    currentComposite = (CompositeDE) deArray[i];
                    if (currentComposite.getXMLTag().compareTo(currentNode.getNodeName()) == 0)
                     {
                      currentComposite.parse(currentNode);  
                     } 
                   continue nodeLoop; 
                  }
                else // its a data element
                 {
                   currentDE = (DataElement) deArray[i];
                   currentDE.set("");  // set fields that have no values
                   if (currentDE.getXMLTag().compareTo(currentNode.getNodeName()) == 0)
                   {
                     
                     for (int nn = 0; nn < cnl.getLength(); nn++)
                      {
	                    cnode = cnl.item(nn);
	                    if (cnode.getNodeType() == Node.TEXT_NODE)
                            currentDE.set(cnode.getNodeValue());
 	                  }  
                      continue nodeLoop; 
                    } 
                 }
            }   
          //virtual else        
         for (i=0; i < templateSegmentVector.size(); i++)
           if (currentNode.getNodeName().compareTo(((TemplateSegment) templateSegmentVector.elementAt(i)).getXMLTag()) == 0)
             {
              currentSegment = new Segment((TemplateSegment) templateSegmentVector.elementAt(i));
              addSegment(currentSegment);
              currentSegment.parse(currentNode);
              continue nodeLoop; // done with this subsegment node
             }   
        throw new OBOEException("Unknown node found " +  currentNode.getNodeName());
       }   
        
   }
   
   private void testDEArraySize(int pos)
   {
    
         if (deArray == null)
            if (myTemplate == null || pos > myTemplate.getTemplateDESize())
               deArray = new Object[pos+1];
            else 
                deArray = new Object[myTemplate.getTemplateDESize()];
         else 
            if (pos >= deArray.length) {// rebuild array
              Object tempArray[] = new Object[deArray.length];
              java.lang.System.arraycopy(deArray, 0, tempArray, 0, deArray.length);
              deArray = new Object[pos+1];
              java.lang.System.arraycopy(tempArray, 0, deArray, 0, tempArray.length);
            }  
              
   }
   
 /**
  * defines a dataelement by the predefined templateDE array
  * @param int position in templateDE array
  * @exception Exception thrown by data element instantiation.
  */ 
   public Object buildDE(int pos)
     {

         testDEArraySize(pos);
         
         TemplateDE tde;
         TemplateComposite tComposite;
         if (myTemplate.isTemplateComposite(pos))
          {
            tComposite = myTemplate.getTemplateComposite(pos);
            deArray[pos] = new CompositeDE(tComposite);
          }  
         else 
         {    
           tde = myTemplate.getTemplateDE(pos);
         
           switch (tde.getType().charAt(0))
           { 
             case 'A':
               if (tde.getIDList()!= null)
                  deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               else 
                  deArray[pos] = new CharDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'C':
               if (tde.getIDList()!= null)
                  deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               else 
                  deArray[pos] = new CharDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'I':
               deArray[pos] = new IDDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag(), tde.getIDList());
               break;
             case 'D':
               deArray[pos] = new DateDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'T':
               deArray[pos] = new TimeDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'N':
               deArray[pos] = new NumericDE(tde.getID(), tde.getSequence(), tde.getType(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             case 'R':
               deArray[pos] = new RealDE(tde.getID(), tde.getSequence(), tde.getDescription(), tde.getRequired(), tde.getMinLength(), tde.getMaxLength(), tde.getXMLTag());
               break;
             default: 
               throw new OBOEException("Unknown dataElement type");
            }  
         } 
      return deArray[pos];
    }    


 /**
  * adds a composite DE to vector
  * @param compositeDE
  */

 public void addCompositeDE(CompositeDE inCDE)
    {
        testDEArraySize(inCDE.getSequence()-1);
        deArray[inCDE.getSequence()-1] = inCDE;
    }    
 /**
  * adds a  DE to vector
  * @param DataElement
  */
  
 public void addDataElement(DataElement inDE)
    {
        testDEArraySize(inDE.getSequence()-1);
        deArray[inDE.getSequence()-1] = inDE;
    }    


/**
 * used to verify if the segment is built correctly. currently not working
 * @return boolean true if segment built correctly
*/
 public boolean validate() 
    {
      boolean returnValidation = true;
      return returnValidation;   
  
    }
    
/**
 * returns the number of subsegments and itself
 * @return int count
*/

public int getSegmentCount()
{
  if (segmentArray == null) return 1;
  int count = 1;
  int i;	
      for (i=0; i < segmentArray.length; i++)
        {
          if (segmentArray[i] != null)
             count++;
		 } 
		 
         
  return count;
}

/**
 returns the number of segments within this segment
 * @return int subsegment count
 */
 
    public int getSegmentSize() { return getSegmentCount()-1;}
    

/**
 * returns the formatted text
 * @param int 
 * <ul>
 * <li> TransactionSet.XML_FORMAT:
 * <li> TransactionSet.X12_FORMAT:
 * <li>  TransactionSet.EDIFACT_FORMAT:
 * <li>otherwise "description: value"
 * </ul>
 */
 
public String getFormattedText(int formatType)
    {

      int i;
      DataElement currentDE;
      CompositeDE currentComposite;
	  Segment currentSegment;
      StringBuffer sbFormattedText = new StringBuffer();

      switch (formatType)
        {
         case TransactionSet.XML_FORMAT:
             sbFormattedText.append('<' +  getXMLTag() + ">\n");
             break;
         case TransactionSet.X12_FORMAT:
             sbFormattedText.append(getID());
             break;
         case TransactionSet.EDIFACT_FORMAT:
             sbFormattedText.append(getID());
             break;
         default:
             sbFormattedText.append(getID() + ": ");
        }
     
      if (deArray != null && deArray[0] != null)
           switch (formatType) {
              case TransactionSet.X12_FORMAT:
               sbFormattedText.append(TransactionSet.X12_FIELD_DELIMITER);
               break;
              case TransactionSet.EDIFACT_FORMAT: 
               sbFormattedText.append(TransactionSet.EDIFACT_FIELD_DELIMITER);
               break;
          }  
          
      for (i=0; i < deArray.length && deArray[i] != null; i++)
        {
          if (isCompositeDE(i))
             {
                currentComposite = (CompositeDE) deArray[i];
                if (currentComposite.get() == null)
                   break;
                if (currentComposite.get().length() == 0)
                   break;
                sbFormattedText.append(currentComposite.getFormattedText(formatType));
                if (i < deArray.length -1 && deArray[i+1] != null)   
                switch (formatType) {
                    case TransactionSet.X12_FORMAT:
                     sbFormattedText.append(TransactionSet.X12_FIELD_DELIMITER);
                     break;
                    case TransactionSet.EDIFACT_FORMAT: 
                     sbFormattedText.append(TransactionSet.EDIFACT_FIELD_DELIMITER);
                     break;
                }  
             }   
          else 
           {
             currentDE = (DataElement) deArray[i];
             if (currentDE.get() == null)
                break;  
             sbFormattedText.append(currentDE.getFormattedText(formatType));
             if (i < deArray.length -1  && deArray[i+1] != null)   
                switch (formatType) {
                    case TransactionSet.X12_FORMAT:
                     sbFormattedText.append(TransactionSet.X12_FIELD_DELIMITER);
                     break;
                    case TransactionSet.EDIFACT_FORMAT: 
                     sbFormattedText.append(TransactionSet.EDIFACT_FIELD_DELIMITER);
                     break;
                }  
           }   
        } 
        
        
     if (deArray!= null && deArray[0] != null)   
      switch (formatType)
        {
         case TransactionSet.X12_FORMAT:
             sbFormattedText.append(TransactionSet.X12_SEGMENT_DELIMITER);
             break;
         case TransactionSet.EDIFACT_FORMAT:
             sbFormattedText.append(TransactionSet.EDIFACT_SEGMENT_DELIMITER);
             break;
         default:
             break;
        }
		
      for (i=0; i < segmentArray.length; i++)
        {
          if (segmentArray[i] == null)
            continue;
          if (isSegment(i)) {
            currentSegment = (Segment) segmentArray[i];
            sbFormattedText.append(currentSegment.getFormattedText(formatType));
          }
          else {
              Vector v = (Vector) segmentArray[i];
              for (int j=0; j<v.size(); j++) {
                currentSegment = (Segment) v.elementAt(j);
                sbFormattedText.append(currentSegment.getFormattedText(formatType));
              }
          }
        } 
        
      switch (formatType)
        {
         case TransactionSet.XML_FORMAT:
             sbFormattedText.append("</" +  getXMLTag() + ">\n");
             break;
         default:
             break;
        }
         
      return new String(sbFormattedText);
  
     
    }
    

/**
 * returns all the data associated with the Segment
 * @return String for all segments and deArray within the Segment
 * @exception Exception thrown when data is missing from a required segment or data element
*/
public String get()
          throws Exception
     {
      StringBuffer sb = new StringBuffer(id);  
      DataElement currentDE; 
      CompositeDE currentCDE; 
      Segment currentSegment; 
      int i;
      for (i=0; i < deArray.length && deArray[i] != null; i++)
       {
          
          if (deArray[i] instanceof DataElement)
            {
              currentDE = (DataElement) deArray[i];          
              if (currentDE.get() == null)
                 break;               
             sb.append((char) '*');
             sb.append(currentDE.get());
            }
          else 
            {
              currentCDE = (CompositeDE) deArray[i];          
              if (currentCDE.get() == null)
                 break;               
             sb.append((char) '<');
             sb.append(currentCDE.get());
            }
       }
      sb.append('\n');  
      
      for (i=0; i < this.getSegmentCount(); i++)
       {
          if (this.isSegment(i)) {
              currentSegment = this.getSegment(i);
              sb.append(currentSegment.get());
          } 
          else if (this.isVector(i))
          { 
            for (int j=0; j < this.getSubsegmentCount(i); j++) {
              currentSegment = this.getSegment(i, j);
              sb.append(currentSegment.get());
            }
          }
          
            
                
        }
  return new String(sb);      
     }   


/**
 * returns the number of defined data element 
 * @return int DataElement count
*/

public int  getDataElementSize() { return deArray.length; }

/**
 * returns a data element by its id
 *
 * @param String  id of the data element
 * @return DataElement
*/

public DataElement getDataElement(String inID)
  {
      int i;
      DataElement currentDE = null;
      
      for (i=0; i < deArray.length; i++)
        {
          if (isDataElement(i)) {
             currentDE = (DataElement) deArray[i];
             if (currentDE.getID().compareTo(inID) == 0)
                break;  
          }    
        } 
      return currentDE;
	}
       
/**
 * returns a data element by its sequence number, not location within vector
 *
 * @param int  sequence number of data element
 * @return DataElement
*/

public DataElement getDataElement(int inSequence)
  {
      if (inSequence >= deArray.length) return null;  
      if (deArray[inSequence] == null) return null;
      if (isDataElement(inSequence)) return (DataElement) deArray[inSequence];
      return null;
	}
       
/**
 * returns a compositeDE by its id
 *
 * @param String  id of the composite
 * @return CompositeDataElement
*/

public CompositeDE getCompositeDE(String inID)
  {
      int i;
      CompositeDE currentDE = null;
     
      for (i=0; i < deArray.length; i++)
        {
          if (isCompositeDE(i)) {
             currentDE = (CompositeDE) deArray[i];
             if (currentDE.getID().compareTo(inID) == 0)
                break;  
          }     
        } 
      return currentDE;
	}
       
       
/**
 * returns a compositeDE by its position
 *
 * @param int position of the composite
 * @return CompositeDataElement
*/

public CompositeDE getCompositeDE(int inSequence)
  {
      if (inSequence >= deArray.length) return null;     
      if (isDataElement(inSequence)) return null;
      return (CompositeDE) deArray[inSequence];
	}
       
       
/**
 * returns a boolean if vector position held by a data element
 *
 * @param int  sequence number of vector element
 * @return DataElement
*/

public boolean isDataElement(int inSequence)
  {
     
      if (inSequence >= deArray.length)
         return false;
      return (deArray[inSequence] instanceof com.americancoders.edi.DataElement);
	}
       
/**
 * returns a boolean if vector position held by a composite
 *
 * @param int  sequence number of vector element
 * @return DataElement
*/

public boolean isCompositeDE(int inSequence)
  {
      if (inSequence >= deArray.length)
         return false;
      return  (deArray[inSequence] instanceof com.americancoders.edi.CompositeDE);
	}
       
       
}
