package com.americancoders.edi;


import java.util.*;
import com.americancoders.*;

/**
 * Segment Container
 *
*<P>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public abstract class SegmentContainer
 {

   protected Vector templateSegmentVector;
   protected Object segmentArray[];
   abstract public String getID();

/**
 * adds a Segment to the segmentArray,  if the Segment is part of the vectorized segment (meaning there can
 * be multiple segments at the same postion) it will be added to the vector
 * @param Segment segment to add
 * @exception OBOEException
 *                Subsegment not defined
 */
 
 
 /** simple routine to build the array based on the number of entries in the template segment vector 
 */
 public void defineSegmentArray()
   {
        TemplateSegment ts;
        segmentArray = new Object[templateSegmentVector.size()];
        for (int i = 0; i < templateSegmentVector.size(); i++)
          {
            ts = (TemplateSegment) templateSegmentVector.elementAt(i);
            if (ts.getOccurs() != 1)
               segmentArray[i] = new Vector();
          }
          
   }
   
   
  public void addSegment(Segment inSegment)
     throws OBOEException
    { int i;
      TemplateSegment tempSeg;
      if (segmentArray == null) defineSegmentArray();
      for (i=0; i < templateSegmentVector.size(); i++)
         {
            tempSeg = (TemplateSegment) templateSegmentVector.elementAt(i);
            if (inSegment.getID().compareTo(tempSeg.getID()) == 0)
              { if (tempSeg.getOccurs() == 1)
                   segmentArray[i] = inSegment;
                else 
                  {Vector v = (Vector) segmentArray[i];
                   if (v == null)
                      {
                        v = new Vector();
                        segmentArray[i] = v;
                      }
                   v.addElement(inSegment);
                  } 
                return; 
              }    
         }
         throw new OBOEException("Subsegment " + inSegment.getID() + " not defined in " + getID());
    }    
   
   
    
/**
 * adds a Segment to the segmentArray by its predefined sequence 
 * @param Segment segment to add
 * @param int sequence number
 * @exception OBOEException
 *                Subsegment not defined at sequence
 *                Subsegment sequence not defined
 */
 
 
  public void addSegment(Segment inSegment, int inPosition)
     throws OBOEException
    { int i;
      if (segmentArray == null)
        defineSegmentArray();
      TemplateSegment tempSeg;
      for (i=0; i < templateSegmentVector.size(); i++)
         {
            tempSeg = (TemplateSegment) templateSegmentVector.elementAt(i);
            if (inSegment.getSequence() == inPosition)
              { 
                if (inSegment.getID().compareTo(tempSeg.getID()) != 0)
                       throw new OBOEException("Subsegment " + inSegment.getID() + " not defined at sequence position " + inPosition + " within " + getID());
                if (tempSeg.getOccurs() == 1) 
                   segmentArray[i] = inSegment;
                else 
                  {Vector v = (Vector) segmentArray[i];
                   if (v == null)
                      {
                        v = new Vector();
                        segmentArray[i] = v;
                      }
                   v.addElement(inSegment);
                  } 
                  return;
              }    
         }
         throw new OBOEException("Subsegment at position " + inPosition + " not defined in " + getID());
         }    
    
/**
 * adds a Segment to a vectorized segment in the segmentArray at a predefined sequence 
 * @param Segment segment to add
 * @param int sequence number
 * @exception OBOEException
 *                segment postion not vectorized
 */
 
 
  public void addSegmentToVector(Segment inSegment, int inPosition)
     throws OBOEException
    { int i;
      TemplateSegment tempSeg;
      if (segmentArray == null) defineSegmentArray();
      
      for (i=0; i < templateSegmentVector.size(); i++)
         {
            tempSeg = (TemplateSegment) templateSegmentVector.elementAt(i);
            if (inSegment.getID().compareTo(tempSeg.getID()) == 0)
              { if (tempSeg.getOccurs() == 1)
        	     throw new OBOEException("Position not held by Vectorized Subsegment");
                else 
                  {Vector v = (Vector) segmentArray[i];
                   if (v == null)
                      {
                        v = new Vector();
                        segmentArray[i] = v;
                      }
                   v.insertElementAt(inSegment, inPosition);
                  } 
                return; 
              }    
         }
         throw new OBOEException("Subsegment " + inSegment.getID() + " not defined in " + getID());
    }    
    
/**
 * sets a Segment to the segmentArray at position in the array,
 * if the Segment is part of the vectorized segment throw exception
 * @param Segment segment to add
 * @param int sequence number
 * @exception OBOEException
 *                out of range
 *                Subsegment vectorized 
 *                Subsegment not defined at sequence
 */
 
 
  public void setSegmentAt(Segment inSegment, int inPosition)
     throws OBOEException
    { 
      TemplateSegment tempSeg;
      
	  if (inPosition < 0 || inPosition > templateSegmentVector.size())
	     throw new OBOEException("Input sequence number out of range for array");
     		 
       if (isVector(inPosition)) 
	     throw new OBOEException("Position held by Vectorized Subsegment");
		 
       tempSeg = (TemplateSegment) templateSegmentVector.elementAt(inPosition);
	   if (tempSeg.getID().compareTo(inSegment.getID()) !=0)
	     throw new OBOEException("Segment not defined at position");
	   
       segmentArray[inPosition] = inSegment;
    }    
         
/**
 * sets a Segment to the vector at a position segmentArray 
 * @param Segment segment to add
 * @param int sequence number
 * @param int position in Vector
 * @exception OBOEException
 *                out of range
 *                Subsegment not vectorized 
 *                Subsegment not defined at sequence
 */
 
 
  public void setSegmentAt(Segment inSegment, int inPosition, int inVectorPosition)
     throws OBOEException
    { 
      TemplateSegment tempSeg;
	  if (inPosition < 0 || inPosition > templateSegmentVector.size())
	     throw new OBOEException("Input sequence number out of range for array");
     		 
       if (isSegment(inPosition)) 
	     throw new OBOEException("Position not held by Vectorized Subsegment");
		 
       tempSeg = (TemplateSegment) templateSegmentVector.elementAt(inPosition);
	   if (tempSeg.getID().compareTo(inSegment.getID()) !=0)
	     throw new OBOEException("Segment not defined at position");
	   
       Vector v = (Vector) segmentArray[inPosition];
	   v.setElementAt(inSegment, inVectorPosition);
     }    
    
/**
 * removes a Segment from the segmentArray by id, does this by setting array entry to null
 * can be used to remove all segments with same id at a vectorized position
 * @param int position
 * @exception OBOEException
 *                     unknown segment id
 */
 
 
  public void removeSegment(String inID)
     throws OBOEException
    { int i;
      TemplateSegment tempSeg;
      for (i=0; i < templateSegmentVector.size(); i++)
         {
            tempSeg = (TemplateSegment)templateSegmentVector.elementAt(i);
            if (tempSeg.getID().compareTo(inID) == 0)
              { 
                segmentArray[i] = null;
                return;
              }    
         }
         throw new OBOEException("Subsegment " + inID + " not defined in " + getID());
    }
/**
 * removes a Segment from the segmentArray by its ID from the subsegment Vector
 * @param int sequence number
 * @param int pos
 * @exception OBOEException
 *                Subsegment not vectorized 
 *                unknown segment id
 */
 
 
  public void removeSegment(String inID, int inPosition)
     throws OBOEException
    { int i;
      TemplateSegment tempSeg;
      for (i=0; i < templateSegmentVector.size(); i++)
         {
            tempSeg = (TemplateSegment)templateSegmentVector.elementAt(i);
            if (tempSeg.getID().compareTo(inID) == 0)
              { 
                if (tempSeg.getOccurs() == 1) 
                   throw new OBOEException("Position not held by Vectorized Subsegment");
                else 
                  {Vector v = (Vector) segmentArray[i];
                   v.remove(inPosition);
                  } 
                return;
              }    
         }
         throw new OBOEException("Subsegment " + inID + " not defined in " + getID());
 }
 
    
/**
 * removes a Segment from the segmentArray at a particular position 
 * can be used to remove all segments with same id at a vectorized position
 * @param int sequence number
 * @exception OBOEException
 *                position number out of range
*/
 
 
  public void removeSegment(int inPosition)
     throws OBOEException
    { 
	  if (inPosition < 0 || inPosition > templateSegmentVector.size())
	     throw new OBOEException("Input sequence number out of range for array");
      segmentArray[inPosition] = null;
    }    

 
/**
 * removes a Segment from the segmentArray at a particular position in the subsegment Vector
 * @param int sequence in array
 * @param int pos in vector
 * @exception OBOEException
 *                position number out of range
 *                Subsegment not vectorized 
 *                Subsegment sequence not defined
 */
 
 
  public void removeSegment(int inPosition, int inVectorPosition)
     throws OBOEException
    { 
	  if (inPosition < 0 || inPosition > templateSegmentVector.size())
	     throw new OBOEException("Input sequence number out of range for array");
		 
      TemplateSegment tempSeg = (TemplateSegment)templateSegmentVector.elementAt(inPosition);
      if (tempSeg.getOccurs() == 1) 
              throw new OBOEException("Position not held by Vectorized Subsegment");
			  
	   Vector v = (Vector) segmentArray[inPosition];	 
       v.remove(inVectorPosition);
            
 }   
    
/**
 * returns a Segment by its position
 * @param int position of the segment
 * @return Segment objects - null indicates segment not set
 * @exception OBOEException
 *                no or not a segment at the position
*/

public Segment getSegment(int inPosition)
        {
         if (segmentArray[inPosition] == null) return null;
         if (!isSegment(inPosition)) 
		   throw new OBOEException("No segment at this position");
         return (Segment) segmentArray[inPosition];
		}
		
		
/**
 * returns the Segment inside the subsegment Vector by its position
 * @param int position of the segment
 * @param int position in the vector
 * @return Vector of Segment objects - null indicates segment not set
 * @exception Exception ID
 * 
*/

public Segment getSegment(int inPosition, int inVectorPosition)
        throws OBOEException 
        {
         if (isSegment(inPosition)) 
               throw new OBOEException("Position not held by Vectorized Subsegment");
         if (inVectorPosition >= ((Vector) segmentArray[inPosition]).size())
            return null;
			  
         return (Segment) ((Vector) segmentArray[inPosition]).elementAt(inVectorPosition);
		}
/**
 * returns a Segment by its ID
 * @param String  id of the segment
 * @return Segment objects - null indicates segment not set
 * @exception OBOEExcetpion
 *                no or not a segment at the position
*/

public Segment getSegment(String ID)
        throws Exception 
        {
         int i;		
		 Segment segment;
         for (i=0; i < segmentArray.length; i++)
		   {
		       if (!isSegment(i)) continue;
			   segment = (Segment) segmentArray[i];
			  
              if (segment.getID().compareTo(ID) == 0)
			     return segment;
			}	 
		 throw new OBOEException ("ID " + ID + " is unknown to segment " + getID());
		}
/** returns the number of subsegments in a vector position 
 * @return int a count - zero if position is not held by a vector 
 */
 
public int getSubsegmentCount(int inPosition) 
  { if (!isVector(inPosition)) return 0;
    return ((Vector) segmentArray[inPosition]).size();
  }  

/** returns true if subsegment type is a segment
 *  if false the subsegment is a vector or position specified is outside range of array
 *  @param int position
 *  @return boolean
 */
 public boolean isSegment(int inPosition)
   {
        if (segmentArray == null) return false;
        if (inPosition < 0 || inPosition >= segmentArray.length) return false;
        if (segmentArray[inPosition] == null) return false;
        if (segmentArray[inPosition] instanceof Vector) return false;
        return true;
        
   }
/** returns true if subsegment type is a vector
 *  if true the subsegment is a segment or position specified is outside range of array
 *  @param int position
 *  @return boolean
 */
 public boolean isVector(int inPosition)
   {
        if (segmentArray == null) return false;
        if (inPosition < 0 || inPosition >= segmentArray.length) return false;
        if (segmentArray[inPosition] == null) return false;
        if (segmentArray[inPosition] instanceof Vector) return true;
        return false;
        
   }

 /**
 * adds a templatesegment to the vector
 * @param TemplateSegment input template
 */
 
  public void addTemplateSegment(TemplateSegment inTemplateSegment)
    {templateSegmentVector.addElement(inTemplateSegment);}
 
/** 
   *returns the number of template segments
   * @return int
   */
   
  public int getTemplateSegmentSize() { return templateSegmentVector.size(); }
     
/**
 * creates a static segment
 * @param String id
 * @return Segment based on this Template
*/

public Segment createSegment(String id)
   throws Exception
    {
      int i;  
      for (i=0; i<templateSegmentVector.size(); i++)
        { if ( ((TemplateSegment) templateSegmentVector.elementAt(i)).getID().compareTo(id) == 0)
            break;
        }
      return new Segment((TemplateSegment) templateSegmentVector.elementAt(i));
    }    

    
/**
 * gets a templatesegment from the vector
 * @param int pos of the templateSegment
 * @return TemplateSegment
 */
 
  public TemplateSegment getTemplateSegment(int inPosition)
     {
      return (TemplateSegment) templateSegmentVector.elementAt(inPosition);
     }
     
/**
 * gets a templatesegment from the vector
 * @param String id of the templateSegment
 * @return TemplateSegment
 */
 
  public TemplateSegment getTemplateSegment(String inID)
     {
        int i;
        for (i=0; i < templateSegmentVector.size(); i++)
           if (inID.compareTo(((TemplateSegment) templateSegmentVector.elementAt(i)).getID()) == 0)
              return (TemplateSegment) templateSegmentVector.elementAt(i);
              
      return null;
     }

 
 
 
 }
