package com.americancoders.edi;

import java.util.*;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import com.americancoders.edi.*;
import com.americancoders.edi.x12.Note_Special_Instruction;
import org.w3c.dom.*;


/**
 * class for Tables
 * a general class for the transaction set's heading detail and summary.
 *
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/
public  class Table extends SegmentContainer implements ITemplateSegmentContainer, Externalizable
 {
  protected String xmlTag = "";
  
  /** constructor takes no parameters
  */
  public Table()
    {
        templateSegmentVector = new Vector();
    }    
	


/**
 * parses an EDI Document
 * 
 * @param ITokenizer input string containing all of the transaction data pretokened by EDIObjects.Tokenizer
  *
 * @exception Exception 
 *                         thrown when the transaction id string is incorrect
 * @exception Exception 
 *                         thrown when an unknown segment id string is foundi
*/
public void parse(ITokenizer TransactionTokenizedString)
                throws Exception
   {
     int i=0;
     Segment currentSegment=null;
     TemplateSegment ts;
     if (TransactionTokenizedString.getCurrentDataElement().compareTo("NTE") == 0)
        {
          currentSegment = Note_Special_Instruction.getInstance();
          addSegment(currentSegment);
          currentSegment.parse(TransactionTokenizedString);
          TransactionTokenizedString.getNextSegment();
          TransactionTokenizedString.getNextDataElement();
          i = -1;
          return;
        }
        
      for (i=0; i < templateSegmentVector.size(); i++)
        {
          ts = (TemplateSegment) templateSegmentVector.elementAt(i);
          if (TransactionTokenizedString.getCurrentDataElement().compareTo(ts.getID()) == 0)
            {
             currentSegment = new Segment((TemplateSegment) templateSegmentVector.elementAt(i));
             addSegment(currentSegment);
             currentSegment.parse(TransactionTokenizedString);
             if (currentSegment.getOccurs() != 1) 
                i--; // reset to look again
            }
          else
           {
             if (ts.areYouAGroup())
               { if (ts.doYouNeedThisSegment(TransactionTokenizedString.getCurrentDataElement()))
                     {               
             currentSegment = new Segment((TemplateSegment) templateSegmentVector.elementAt(i));
             addSegment(currentSegment);
             currentSegment.parse(TransactionTokenizedString);
             if (currentSegment.getOccurs() != 1) 
                i--; // reset to look again
                     } 
               } 
           }
        }   
   }
   
/**
 * parses a XML EDI Document
 * 
 * @param Node  DOM XML Node
 *
 * @exception Exception 
 *                         thrown when the transaction id string is incorrect
 * @exception Exception 
 *                         thrown when an unknown segment id string is foundi
*/
public void parse(Node node)
                throws Exception
   {

      int i=0;
      Segment currentSegment=null;
      
      for (i=0; i < templateSegmentVector.size(); i++)
        if (node.getNodeName().compareTo(((TemplateSegment) templateSegmentVector.elementAt(i)).getXMLTag()) == 0)
        {
          currentSegment = new Segment((TemplateSegment) templateSegmentVector.elementAt(i));
          addSegment(currentSegment);
          currentSegment.parse(node);
          return;
        }
        
      throw new OBOEException("Unknown node passed to parser" + node.getNodeName());
 

   }
   
/**
 * tests if the node is part of this table
 * 
 * @param Node  XML Node
 * @return boolean 
 *
 * @exception Exception 
 *                         thrown when the transaction id string is incorrect
 * @exception Exception 
 *                         thrown when an unknown segment id string is foundi
*/
public boolean doYouWantThisNode(Node node)
                throws Exception
   {

      for (int i=0; i < templateSegmentVector.size(); i++)
	   { 
        if (node.getNodeName().compareTo(((TemplateSegment) templateSegmentVector.elementAt(i)).getXMLTag()) == 0)
          return true;
	   }	  
	   
	  return false;	  
   }
   
 /** 
 * validates if the table is defined correctly
 * @return boolean true if built correctly
 *
 */

  public  boolean validate()
    { 
      boolean validateResponse = true;
        return validateResponse;
    }

/**
 * returns the Table in an EDI document format
 * @return String
 */

    
  public  String get() throws Exception  
    {
      StringBuffer sb = new StringBuffer();  
      Segment currentSegment; 
      int i;
      for (i=0; i < segmentArray.length; i++)
       {
          if (segmentArray[i] == null) continue;
          if (isSegment(i)) {
              currentSegment = (Segment) segmentArray[i];
              sb.append(currentSegment.get());
          } 
          else
          { Vector v = (Vector) segmentArray[i];
            for (int j=0; j<v.size(); j++) {
              currentSegment = (Segment) v.elementAt(j);
              sb.append(currentSegment.get());
            }
          }
       }  
            
  return new String(sb);      
 }

    
/**
 * returns the total number of segments in use within the the table
 * this includes the count of subsegments
 * @return int
 */
 
  public int getSegmentCount()
    {
      int i;
      int count = 0;
      for (i=0; i < segmentArray.length; i++)
        {
          if (segmentArray[i] != null)
             count++;
		 } 
      return count;   
    }
    
    
 /** returns a preformatted string
  * @param int type - TransactionSet.XML_FORMAT returns a string of the entire 
  * table.
  * default is plain text
  * @return String
  */
  
   public String getFormattedText(int formatType)
     {
      int i;
      StringBuffer sbFormattedText = new StringBuffer();
      Segment currentSegment;
      
      for (i=0; i < segmentArray.length; i++)
        {
          if (segmentArray[i] == null)
            continue;
          if (isSegment(i)) {
            currentSegment = (Segment) segmentArray[i];
            sbFormattedText.append(currentSegment.getFormattedText(formatType));
          }
          else {
              Vector v = (Vector) segmentArray[i];
              for (int j=0; j<v.size(); j++) {
                currentSegment = (Segment) v.elementAt(j);
                sbFormattedText.append(currentSegment.getFormattedText(formatType));
              }
          }
        } 
         
      return new String(sbFormattedText);
  
     }     

 /** returns the table id, since there are no tables id it returns a ""
 * @return String
 */
public String getID()
  { return "";}
  
/** sets the xml tag field 
 * @param String xml tag value
 */

public void  setXMLTag(String inXMLTag){xmlTag = inXMLTag;}
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
public String getXMLTag() { return xmlTag; }
 
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
   {
    xmlTag = in.readUTF();
    templateSegmentVector = (Vector) in.readObject();
    defineSegmentArray();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(xmlTag);
    out.writeObject(templateSegmentVector);
   }
         
  

}
        
