package com.americancoders.edi;


import java.util.Vector;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

/* Template Composite holds preliminary structure of Composites 
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
 @author Joe McVerry, American Coders Ltd.
 @version 1.4.1
*/
public class TemplateComposite implements IIdentifier, Externalizable
{
   protected String id;
   protected String name;
   protected int sequence = 0;
   protected String description;
   protected String xmlTag;
    
   private Vector templateDEVector;

  public TemplateComposite() {}         



/**
 * TemplateCompositeVector, there are two flavors of composites Templates and regular
 * <P> templates are used to define a composite dynamically 
 * <p> and are used to build the static form of composites
 * <p> contains template data element
 * @param string inID composite id
 * @param string inName composite name
 * @param int composite sequence 
 * @param String composite description
 * @param String xml tag
*/

   public TemplateComposite(String inId, String inName, int inSequence, String inDescription, String inXMLTag)
     {
        setID(inId);
        setName(inName);
        setSequence(inSequence);
        setDescription(inDescription);
        templateDEVector = new Vector();
        setXMLTag(inXMLTag);
     }   
	 
/**
 * TemplateCompositeVector, there are two flavors of composites Templates and regular
 * <P> templates are used to define a composite dynamically 
 * <p> and are used to build the static form of composites
 * <p> contains template data elements and secondary composites
 * @param string inID composite id
 * @param string inName composite name
 * @param int composite sequence 
 * @param String composite description
 * @param int the number of times the composite occurs
*/

   public TemplateComposite(String inId, String inName, int inSequence, String inDescription)
     {
        setID(inId);
        setName(inName);
        setSequence(inSequence);
        setDescription(inDescription);
        templateDEVector = new Vector();
     }   

  public int getTemplateDESize() { return templateDEVector.size(); }
	 
/**
 * sets TemplateComposite id
 * @param string inID composite id
*/
   public void setID(String inID) {id = new String(inID); }
   
/*8
 * sets TemplateComposite name
 * @param string inID composite name
*/
   public void setName(String inName) {name = new String(inName); }
   
/**
 * gets TemplateComposite id
 * @returns string inID composite id
*/
   public String getID() { return id; }
/**
 * gets TemplateComposite name
 * @returns string inID composite name
*/
   public String getName() { return name; }
   
/**
 * sets sequence 
 * @param int sequence
*/
   public void setSequence(int inSequence){ sequence = inSequence; }
   
   
/**
 * gets sequence 
 * @return int sequence
*/
   public int getSequence(){ return sequence; }
    
/**
 * sets Description for the Segmemt
 * @param String
 *
*/
    public void setDescription(String inDesc) {description = inDesc;};

/**
 * returns the Description for the Segment
 * @return String
*/
    public String getDescription() {return description;};

/** sets the xmltag field
 * @param String
 */


 public void  setXMLTag(String inXMLTag)
 {
    xmlTag = inXMLTag;
 }
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
  
 public String getXMLTag()
 {
    if (xmlTag == null) return getID();
    return xmlTag;
 }
 
/**
 * adds to TemplateDE container
 * @param TemplateDE
*/
   public void addDataElement(TemplateDE inTemplateDE)   
       { templateDEVector.addElement(inTemplateDE); }
       
/**
 * gets TemplateDE from TemplateDE container by position
 * @param int position in container
 * @return TemplateDE
*/
   public TemplateDE getTemplateDE(int at) 
        {return (TemplateDE) templateDEVector.elementAt(at); }
        
/**
 * gets TemplateDE from TemplateDE container by ID
 * @param String id of TemplateDE in container
 * @return TemplateDE null if not found
*/
   public TemplateDE getTemplateDE(String id)
        { 
             TemplateDE tDE;
             for (int i = 0 ; i < templateDEVector.size(); i++) {
                   tDE = (TemplateDE) templateDEVector.elementAt(i);
                   if (tDE.getID().compareTo(id) == 0) return tDE;
                 } /* endif */
              return null;
         }

/**
 * creates a static composite
 * @param String id
 * @return Segment based on this Template
*/
 public CompositeDE createComposite(String id)
    throws Exception
    {
      int i;
      TemplateComposite tC=null;
      return new CompositeDE(tC);
    }    
 /** 
  * routine to ask if it uses a dataelement 
  * @param String id
  * @param int whereToStartLooking, prevents duplicate DE's from getting only the first
  * @return boolen true the composite id is part of this group
 */
  public int doYouUseThisXMLElement(String inXML, int startAt)
    {
      IIdentifier iid;
      for (int i=startAt+1; i<templateDEVector.size(); i++)
        {   iid = (IIdentifier) templateDEVector.elementAt(i);
            if ( iid.getXMLTag().compareTo(inXML) == 0)
                return i;
        }
     return -1;
   }

/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
   public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
     {
    id = in.readUTF();
    name = in.readUTF();
    sequence = in.readInt();
    description = in.readUTF();
    xmlTag = in.readUTF();
    templateDEVector = (Vector) in.readObject();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
   public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(id);
    out.writeUTF(name);
    out.writeInt(sequence);
    out.writeUTF(description);
    
    out.writeUTF(xmlTag);
    out.writeObject(templateDEVector);
   }
}
