package com.americancoders.edi;

import java.util.Vector;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

/**
 * class for Template Data Elements
 * <br> template de's are dynamic definitions for de's
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
 *
*/

public class TemplateDE implements IIdentifier, Externalizable
 {
    protected  String type = "";
    protected  int sequence;
    protected  String id = "";
    protected  String name = "";
    protected  char required = ' ';
    protected  String referenceId = "";
    protected  int minLength, maxLength;
    protected  String description;
    protected  String xmlTag;
    private IDList idList;
        
/**
 * constructs the Data Element type used for serialization 
 */
 
public TemplateDE() {}
/**
 * constructs the Data Element type
 * @param String   id
 * @param String   name
 * @param int sequence within segment or composite
 * @param String   type
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 * @param String idListFile xml file name containing List information
 *
*/
public TemplateDE(String inID, String inName, int inSequence, String inType, char inRequired, String inDesc, int inMinLength, int inMaxLength, String inXMLTag, IDList inIDList)
{
  setID(inID);
  setName(inName);
  setSequence(inSequence);
  setType(inType);
  setRequired(inRequired);
  setDescription(inDesc);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
  setXMLTag(inXMLTag);
  setIDList(inIDList);
}

/**
 * constructs the Data Element type
 * @param String   id
 * @param String   name
 * @param int sequence within segment or composite
 * @param String   type
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag
 *
*/
public TemplateDE(String inID, String inName, int inSequence, String inType, char inRequired, String inDesc, int inMinLength, int inMaxLength, String inXMLTag)
{
  setID(inID);
  setName(inName);
  setSequence(inSequence);
  setType(inType);
  setRequired(inRequired);
  setDescription(inDesc);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
  setXMLTag(inXMLTag);
}

/**
 * constructs the Data Element type
 * @param String   id
 * @param String   name
 * @param int sequence within segment or composite
 * @param String   type
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 *
*/
public TemplateDE(String inID, String inName, int inSequence, String inType, char inRequired, String inDesc, int inMinLength, int inMaxLength)
{
  setID(inID);
  setName(inName);
  setSequence(inSequence);
  setType(inType);
  setRequired(inRequired);
  setDescription(inDesc);
  setMinLength(inMinLength);
  setMaxLength(inMaxLength);
}

/**
 * sets the Data Element type
 * @param String 
 *
*/
    public void setType(String inType) { type = inType; }
	
	
/**
 * gets the Data Element type
 * @return String
 *
*/
    public String getType() { return type; }

/**
 * sets the Data Element required
 * @param char
 *
*/
    public void setRequired(char inRequired) { required = inRequired; }
	
	
/**
 * gets the Data Element required
 * @return char
 *
*/
    public char getRequired() { return required; }

/**
 * sets the Data Element id
 * @param String 
 *
*/
    public void setID(String inID) { id = inID; }

/**
 * gets the Data Element id
 * @return String
 *
*/
    public String getID() { return id; }

/**
 * sets the Data Element name
 * @param String 
 *
*/
    public void setName(String inname) { name = inname; }
	
	
/**
 * gets the Data Element name
 * @return String
 *
*/
    public String getName() { return name; }

/**
 * sets the Data Element sequence
 * @param int
 *
*/
    public void setSequence(int inSequence) { sequence = inSequence; }

/**
 * gets the Data Element sequence
 * @return int
 *
*/
    public int getSequence() { return sequence; }

/**
 * sets the minimum length for the Data Element
 * @param int
 *
*/
    public void setMinLength(int inMinLength) { minLength = inMinLength; }
/**
 * sets the maximum length for the Data Element
 * @param int
 *
*/
    public void setMaxLength(int inMaxLength) { maxLength = inMaxLength;  }
/**
 * gets the minimum length for the Data Element
 * @return int
 *
*/
    public int getMinLength() { return minLength; }
	
	
/**
 * gets the maximum length for the Data Element
 * @return int
 *
*/
    public int getMaxLength() { return maxLength;  }


/**
 * sets Description for the Data Element
 * @param String
 *
*/
    public void setDescription(String inDesc) {description = inDesc;};

/**
 * returns the Description for the Data Element
 * @return String
*/
    public String getDescription() {return description;};

 /**
  * sets the xml tag field 
  * @param String tag value
  */
  
 public void  setXMLTag(String inXMLTag)
 {
    xmlTag = inXMLTag;
 }
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
  
 public String getXMLTag()
 {
    if (xmlTag == null) return getID();
    return xmlTag;
 }
 
  
 
  /** sets the idList Object
   *  @param IDList
   */
 
  public void setIDList(IDList inIdList)
   {  idList=inIdList; }   


  /** gets the idList Object
   *  @return IDList
   */
  public IDList getIDList()
    {  return idList; }
 
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
   public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
     {
    type = in.readUTF();
    sequence = in.readInt();
    id = in.readUTF();
    name = in.readUTF();
    required = in.readChar();
    referenceId = in.readUTF();
    minLength = in.readInt();
    maxLength = in.readInt();
    description = in.readUTF();
    xmlTag = in.readUTF();
    idList = (IDList) in.readObject();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
   public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(type);
    out.writeInt(sequence);
    out.writeUTF(id);
    out.writeUTF(name);
    out.writeChar(required);
    out.writeUTF(referenceId);
    out.writeInt(minLength);
    out.writeInt(maxLength);
    out.writeUTF(description);
    out.writeUTF(xmlTag);
    out.writeObject(idList);
   }
   
}

