package com.americancoders.edi;

import java.util.Vector;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;

/**
 * TemplateSegment holds preliminary segment structure
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/
public class TemplateSegment implements ITemplateSegmentContainer, Externalizable
{
   protected String id;
   protected String name;
   protected int sequence = 0;
   protected String description;
   protected String xmlTag;
   protected int occurs;
   protected char required;
    
   private Vector templateSegmentVector;
   private Vector templateDEVector;


/**
 * templateSegment constructor used by externalize mehtods
 */
  public TemplateSegment() {}         

/**
 * templateSegmentVector, there are two flavors of segments Templates and regular
 * <P> templates are used to define a segment dynamically 
 * <p> and are used to build the static form of segments
 * <p> contains template data elements and secondary segments
 * @param string inID segment id
 * @param string inName segment name
 * @param int segment sequence 
 * @param String segment description
 * @param int the number of times the segment occurs
 * @param char requirement 
 * @param String xml tag
*/


   public TemplateSegment(String inId, String inName, int inSequence, String inDescription, int inOccurs, char inRequired, String inXMLTag)
     {
        setID(inId);
        setName(inName);
        setSequence(inSequence);
        setDescription(inDescription);
        templateSegmentVector = new Vector();
        templateDEVector = new Vector();
        setOccurs(inOccurs);
        setXMLTag(inXMLTag);
        setRequired(inRequired);
     }   
	 
/**
 * templateSegmentVector, there are two flavors of segments Templates and regular
 * <P> templates are used to define a segment dynamically 
 * <p> and are used to build the static form of segments
 * <p> contains template data elements and secondary segments
 * @param string inID segment id
 * @param string inName segment name
 * @param int segment sequence 
 * @param String segment description
 * @param int the number of times the segment occurs
 * @param char requirement 
*/

   public TemplateSegment(String inId, String inName, int inSequence, String inDescription, int inOccurs, char inRequired )
     {
        setID(inId);
        setName(inName);
        setSequence(inSequence);
        setDescription(inDescription);
        templateSegmentVector = new Vector();
        templateDEVector = new Vector();
        setOccurs(inOccurs);
        setRequired(inRequired);
     }   

/** returns the number of templatesegments
 * @return int
 */
  public int getTemplateSegmentSize() { return templateSegmentVector.size(); }
  
  
/** returns the  number of templateDEs
 * @return int
 */
  public int getTemplateDESize() { return templateDEVector.size(); }
	 
/**
 * sets TemplateSegment id
 * @param string inID segment id
*/
   public void setID(String inID) {id = new String(inID); }
   
/**
 * sets TemplateSegment name
 * @param string inName
*/
   public void setName(String inName) {name = new String(inName); }
   
/**
 * gets TemplateSegment id
 * @returns string 
*/
   public String getID() { return id; }
   
/**
 * gets TemplateSegment name
 * @returns string name
*/
   public String getName() { return name; }
   
/**
 * sets sequence id
 * @param int sequence
*/
   public void setSequence(int inSequence){ sequence = inSequence; }
   
   
/**
 * gets sequence id
 * @return int sequence
*/
   public int getSequence(){ return sequence; }
    
/**
 * sets Description for the Segmemt
 *
*/
    public void setDescription(String inDesc) {description = inDesc;};

/**
 * returns the Description for the Segment
 * @return String
*/
    public String getDescription() {return description;};

/**
 * sets occurance value
 * @param int occurs
*/
   public void setOccurs(int inOccurs){ occurs = inOccurs; }
   
   
/**
 * gets occurance value
 * @return int occurs
*/
   public int getOccurs(){ return occurs; }

/**
 * sets required value
 * @param char required
*/
   public void setRequired(char inRequired){ required = inRequired; }
   
   
/**
 * gets required flag
 * @return char required
*/
   public char getRequired(){ return required; }

/** sets the xmlTag field
 * @param String
 */

 public void  setXMLTag(String inXMLTag)
 {
    xmlTag = inXMLTag;
 }
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
  
 public String getXMLTag()
 {
    return xmlTag;
 }
 
 
 
/**
 * adds to secondary TemplateSegment container
 * @param TemplateSegment 
*/
   public void addTemplateSegment(TemplateSegment inTemplateSegment)   
       { templateSegmentVector.addElement(inTemplateSegment); }
	   
	   
       
/**
 * returns secondary TemplateSegment from container
 * @param int position in segment container
 * @return TemplateSegment
*/
   public TemplateSegment getTemplateSegment(int at) 
        {return (TemplateSegment) templateSegmentVector.elementAt(at); }
        
/**
 * returns secondary TemplateSegment from container
 * @param String secondary segment id
 * @return TemplateSegment
*/
   public TemplateSegment getTemplateSegment(String id)
        { 
             TemplateSegment tSEG;
             for (int i = 0 ; i < templateSegmentVector.size(); i++) {
                   tSEG = (TemplateSegment) templateSegmentVector.elementAt(i);
                   if (tSEG.getID().compareTo(id) == 0) return tSEG;
                 } /* endif */
              return null;
         }


/**
 * returns true if id is one of the secondary TemplateSegment's id
 * @param String id to find
 * @return boolean found or not found
*/
   public boolean validSegment(String id)
        { 
             TemplateSegment tSEG;
             for (int i = 0 ; i < templateSegmentVector.size(); i++) {
                   tSEG = (TemplateSegment) templateSegmentVector.elementAt(i);
                   if (tSEG.getID().compareTo(id) == 0) return true;
                 } /* endif */
              return false;
         }


/**
 * adds to TemplateDE container
 * @param TemplateComposite
*/
   public void addCompositeDE(TemplateComposite inTemplateComposite)   
       { templateDEVector.addElement(inTemplateComposite); }
/**
 * adds to TemplateDE container
 * @param TemplateDE
*/
   public void addDataElement(TemplateDE inTemplateDE)   
       { templateDEVector.addElement(inTemplateDE); }
       
/**
 * tests if the element at a position is a TemplateComposite 
 * @param int position in container
 * @return boolean
*/
   public boolean isTemplateComposite(int at) 
       {return (templateDEVector.elementAt(at) instanceof com.americancoders.edi.TemplateComposite); }
	   
	   
/**
 * gets TemplateComposite from TemplateDE container
 * @param int position in container
 * @return TemplateComposite
*/
   public TemplateComposite getTemplateComposite(int at) 
        {return (TemplateComposite) templateDEVector.elementAt(at); }
		
        
/**
 * gets TemplateComposite from TemplateDE container
 * @param String id of TemplateComposite in container
 * @return TemplateComposite
*/
   public TemplateComposite getTemplateComposite(String id)
        { 
             TemplateComposite tComposite;
             for (int i = 0 ; i < templateDEVector.size(); i++) {
                   tComposite = (TemplateComposite) templateDEVector.elementAt(i);
                   if (tComposite.getID().compareTo(id) == 0) return tComposite;
                 } /* endif */
              return null;
         }

/**
 * gets TemplateDE from TemplateDE container
 * @param int position in container
 * @return TemplateDE
*/
   public TemplateDE getTemplateDE(int at) 
        {return (TemplateDE) templateDEVector.elementAt(at); }
        
/**
 * gets TemplateDE from TemplateDE container
 * @param String id of TemplateDE in container
 * @return TemplateDE
*/
   public TemplateDE getTemplateDE(String id)
        { 
             TemplateDE tDE;
             for (int i = 0 ; i < templateDEVector.size(); i++) {
                   tDE = (TemplateDE) templateDEVector.elementAt(i);
                   if (tDE.getID().compareTo(id) == 0) return tDE;
                 } /* endif */
              return null;
         }

/**
 * creates a static segment
 * @param String id
 * @return Segment based on this Template
*/
 public Segment createSegment(String id)
    throws Exception
     {
      int i;
      TemplateSegment tSG=null;
      for (i=0; i<templateSegmentVector.size(); i++)
        {   tSG = (TemplateSegment) templateSegmentVector.elementAt(i);
            if ( tSG.getID().compareTo(id) == 0)
                break;
        }
      return new Segment(tSG);
    }    

/**
 * Simple routine to determine if the segment contains no data elements
 * <br> if it returns true then it is a EDIFact Group
 * 
 * @return boolean true - EDIFact group, contains no data elements
*/
  public boolean areYouAGroup()
    {
    return templateDEVector.size() == 0;
    }
    
 /** 
  * routine to ask a group if it uses a segment 
  * @param String id
  * @return boolen true the segment id is part of this group
 */
  public boolean doYouNeedThisSegment(String inID)
    {
      TemplateSegment tsg;
      for (int i=0; i<templateSegmentVector.size(); i++)
        {   tsg = (TemplateSegment) templateSegmentVector.elementAt(i);
            if ( tsg.getID().compareTo(inID) == 0)
                return true;
        }
     return false;
   }
 /** 
  * routine to ask if it uses a dataelement 
  * @param String id
  * @return boolen true the segment id is part of this group
 */
  public int doYouUseThisElement(String inID)
    {
      IIdentifier iid;
      for (int i=0; i<templateDEVector.size(); i++)
        {   iid = (IIdentifier) templateDEVector.elementAt(i);
            if ( iid.getID().compareTo(inID) == 0)
                return i;
        }
     return -1;
   }
 /** 
  * routine to ask if it uses a dataelement 
  * @param String id
  * @param int whereToStartLooking, prevents duplicate DE's from getting only the first
  * @return boolen true the segment id is part of this group
 */
  public int doYouUseThisXMLElement(String inXML, int startAt)
    {
      IIdentifier iid;
      for (int i=startAt+1; i<templateDEVector.size(); i++)
        {   iid = (IIdentifier) templateDEVector.elementAt(i);
            if ( iid.getXMLTag().compareTo(inXML) == 0)
                return i;
        }
     return -1;
   }

/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
   public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
     {
    id = in.readUTF();
    name = in.readUTF();
    sequence = in.readInt();
    description = in.readUTF();
    xmlTag = in.readUTF();
    occurs = in.readInt();
    required = in.readChar();
    
   templateSegmentVector = (Vector) in.readObject();
   templateDEVector = (Vector) in.readObject();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
   public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(id);
    out.writeUTF(name);
    out.writeInt(sequence);
    out.writeUTF(description);
    
    out.writeUTF(xmlTag);
    out.writeInt(occurs);
    out.writeChar(required);

    out.writeObject(templateSegmentVector);
    out.writeObject(templateDEVector);
   }
        
}
