package com.americancoders.edi;

import java.util.*;
import java.text.*;


/**
 * class for Data Elements defined as Time
 *
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/
public class TimeDE extends DataElement
 {
    private Calendar calendar;

/**
 * constructs a TimeDE object 
 * @param String ID
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
*/
public TimeDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        calendar = Calendar.getInstance();
     
	}

/**
 * constructs a TimeDE object 
 * @param String ID
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
public TimeDE(String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        calendar = Calendar.getInstance();
     }

/**
 * constructs TimeDE object using an input string in the format HHMMSS where
 * <br> some time fields are 4 characters long others maybe 6 or more.
 * <UL> 
 * <LI> HH is hour 
 * <LI> MM is minutes
 * <LI> SS is seconds 
 *</UL>
 * @param inTime    String field in format HHMMSS
 * @param String ID
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
 * @param String xml tag value
*/
public TimeDE(String inTime, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength, String inXMLTag)
    {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength, inXMLTag);
        calendar = Calendar.getInstance();
        set(inTime);
     }

/**
 * constructs TimeDE object using an input string in the format HHMMSS where
 * <br> some time fields are 4 characters long others maybe 6 or more.
 * <UL> 
 * <LI> HH is hour 
 * <LI> MM is minutes
 * <LI> SS is seconds 
 *</UL>
 * @param inTime              String field in format HHMMSS
 * @param String ID
 * @param int sequence number within structure
 * @param String Description
 * @param char required
 * @param int minimum allowed length
 * @param int maximum allowed length
*/
public TimeDE(String inTime, String inID, int inSequence, String inDesc, char inRequired, int inMinLength, int inMaxLength)
    {
        super(inID, inSequence, inDesc, inRequired, inMinLength, inMaxLength);
        calendar = Calendar.getInstance();
        set(inTime);
     }

/**
 * returns the date stored in HHMMSS format  where
 * <UL> 
 * <LI> HH is hour 
 * <LI> MM is minutes
 * <LI> SS is seconds
 *</UL>

 * @return String in format HHMMSS
*/
    public String get()
      {

        if (value == null)
          return null;
          
        if (value.length() == 0) 
          return "";

        SimpleDateFormat sdf = new SimpleDateFormat("HHmm");
        value = sdf.format(calendar.getTime());
        return value.substring(0, minLength);
      }

/**
 * returns the date field as stored
 * @param int field identifier
 * <BR>use the java.util.Calendar fields:
 * <OL>
 * <LI> HOUR
 * <LI> MINUTE
 * <LI> SECOND
 * </OL>
 * @see java.util.Calendar
 * @return String in format HHMMSS
*/
    public int get(int field)
      {
       return calendar.get(field);
      }

/**
 * sets the value stored using an input string in the format HHMMSS where
 * <UL> 
 * <LI> HH is hour 
 * <LI> MM is minutes
 * <LI> SS is seconds, 
 *</UL>
 * @param inTime              String field in format HHMMSS
*/
    public void set(String inTime)
      {
         if (inTime.length() == 0)
          {
           value = "";
           return;
          }
         calendar.set(Calendar.HOUR,  Integer.parseInt(inTime.substring(0,2)));
         calendar.set(Calendar.MINUTE,  Integer.parseInt(inTime.substring(2,4)));
         if (inTime.length() == 4) 
            calendar.set(Calendar.SECOND,  0);
         else
            calendar.set(Calendar.SECOND,  Integer.parseInt(inTime.substring(4,6)));
          
         value = new String(inTime);
      }

/**
 * sets a specific part of the date field
 * @param int field identifier
 * <BR>use the java.util.Calendar fields:
 * <OL>
 * <LI> HOUR
 * <LI> MINUTE
 * <LI> SECOND
 * </OL>
 * @param int value
 * @see java.util.Calendar
*/
    public void set(int field, int value)
      {
        calendar.set(field, value);
      }

 /**
  * formats text of data element
  * <br> Short Description of DataElement is defined in the class
  * <br> value is the current value set in the object
  * @param int formatType
  * @return String formatted string
  */

 public String getFormattedText(int formatType)
    {

    SimpleDateFormat sdf = new SimpleDateFormat("hh:mm aa");
    if (value == null)
      return null;
    StringBuffer sb = new StringBuffer();
        switch (formatType)
     {
      case TransactionSet.XML_FORMAT:
        sb.append("<" + getXMLTag() + ">" + sdf.format(calendar.getTime()) + "</" + getXMLTag() + ">\n");
        break;
      case TransactionSet.X12_FORMAT:
      case TransactionSet.EDIFACT_FORMAT:
           sb.append(value);
           break;
      default:
        sb.append("\t" + getDescription() + ": " + sdf.format(calendar.getTime()) + "\n");
     }   

    return new String(sb);
    }


}
