package com.americancoders.edi;

import java.util.*;


/**
 *  class to assist in tokenizing input transaction sets
 *  <br>
 *  field seperator uses 3rd byte of input string 
 *  segment separator uses 16th field + 1 byte field, if it sees a cr character then checks for a lf character and then assumes a '\n' character
 *
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class Tokenizer implements ITokenizer
 {

   protected String type;
   protected StringTokenizer transactionSetString;
   protected String currentTokenStringToken;
   protected DataTokenizer currentTokenString;
   protected String currentDataElement;
   protected boolean dataElementReady;
   protected int segmentPos  = -1;
   protected int dataelementPos = -1;
   protected String tokenCharacter;
   protected String subelementTokenCharacter;
   protected char tokenGroups[] = {' '};

/**
 *  builds the parsing object for a transaction set
 *  @param String input string
 *  @param String type (X12 | EDIFACT)
 *
*/
   public Tokenizer(String str, String inType) 
      throws Exception
     {
     type = inType;   
     if (type.compareTo("X12") == 0)
        {
        char tokenChars[] = new char[1];
        int tokencnt=0;
        for (int i=0; i!=str.length(); i++) 
          {
            if (str.charAt(i) == str.charAt(3))
              {
                tokencnt++;
                if (tokencnt == 16)
                   {
                    
                      tokenGroups[0] = str.charAt(i);
                      tokenChars[0] = str.charAt(i+1);
                      subelementTokenCharacter = new String(tokenChars);
		
                      tokenChars[0] = str.charAt(i+2);
                      if (str.charAt(i+2) == 13)
		                 if (str.charAt(i+3) == 10)
                           tokenChars[0] = '\n';
                           
                      tokenCharacter = new String(tokenChars);
                      
                      break;
                   }
              }
          }   
		
        transactionSetString = new StringTokenizer(str, tokenCharacter);
		
		tokenChars = new char[1];
        tokenChars[0] = str.charAt(3);
        tokenCharacter = new String(tokenChars);
		
        dataElementReady = false;
     }
     else 
     if (type.compareTo("EDIFACT") == 0){
        
        if (str.startsWith("UNA"))
          {  
                char tokenChars[] = new char[1];
                tokenChars[0] = str.charAt(8);
     
                tokenCharacter = new String(tokenChars);
                transactionSetString = new StringTokenizer(str, tokenCharacter);
		
                tokenChars[0] = str.charAt(4);
                tokenCharacter = new String(tokenChars);
        
                tokenGroups[0] = str.charAt(3);
		  }
		else // use defaults
		  {
		   
		   transactionSetString = new StringTokenizer(str, TransactionSet.EDIFACT_SEGMENT_DELIMITER);
		   tokenCharacter = new String(TransactionSet.EDIFACT_FIELD_DELIMITER);
		   tokenGroups[0] = TransactionSet.EDIFACT_GROUP_DELIMITER.charAt(0);
		  }
        dataElementReady = false;
     }
     else
       throw new OBOEException("Parsing failed to tokeninize unknown type: " + type);
     }

/**
 *  returns a count of segments in transaction set
 *  @return int the number of segments
 *
*/
   public int countSegments()
    {
       return transactionSetString.countTokens();
    }

/**
 *  returns the next segment to be parsed
 *  @return IDataTokenizer the next segment
 *
*/
   public IDataTokenizer getNextSegment()
     {
        segmentPos++;
        dataelementPos=-1;
        try {
             currentTokenStringToken = (String) transactionSetString.nextElement();
        }
        catch (Exception e1) {return null;}
        int pos = currentTokenStringToken.indexOf('\r');
        if (pos > -1)
            currentTokenStringToken = currentTokenStringToken.substring(0, pos);
        currentTokenString = new DataTokenizer(currentTokenStringToken , tokenCharacter);
        dataElementReady = false;
        return currentTokenString;
     }       


/**
 *  indicates if there are more segments to parse
 *  @return boolean  more to parse?
 *
*/
   public boolean hasMoreSegments()
     {
        return transactionSetString.hasMoreElements();
     }

/**
 *  resets the current segment to the beginning - an inverse look-ahead method
 *  @return IDataTokenizer the next segment
 *
*/
   public IDataTokenizer resetSegment()
     {
        dataelementPos = -1;;
        currentTokenString = new DataTokenizer(currentTokenStringToken , tokenCharacter);
        getNextDataElement();
        return currentTokenString;
     }

/**
 *  returns the position (line number?) of the current segment being tokenized. value
 *  is relative to zero
 *  @return int the current segment position (line number)
 *
*/
   public int getSegmentPos()
     {
        return segmentPos;
     }

/**
 *  returns a count of dataelements in segment
 *  @return int the number of data elements
 *
*/
   public int countDataElements()
     {
        return currentTokenString.countTokens();
     }

/**
 *  returns the currently parsed data element.  It does not return
 *  a data element object but a value.
 *  @return String current value of data element
 *
*/
   public String getCurrentDataElement()
     {
        return currentDataElement;
     }

/**
 *  returns the next data element to be parsed
 *  @return String current value of data element
 *
*/
   public String getNextDataElement()
      {

         if  (currentTokenString.hasMoreElements())
           {
             currentDataElement = currentTokenString.nextToken();
             dataelementPos++;
             dataElementReady = true;
           }
        else
           {
            currentDataElement = "";
            dataElementReady = false;
           }

        return currentDataElement;

      }

/**
 *  indicates if there are more data elements to parse
 *  @return boolean  more to parse?
 *
*/
   public boolean hasMoreDataElements()
      {
         return currentTokenString.hasMoreElements();
      }


/**
 *  indicates if there is data element to parse
 *  @return boolean  data element ready?
 *
*/
   public boolean isThereADataElement()
      {
         return dataElementReady;
      }

/**
 *  returns the position  of the current dataelement being tokenized.
 *  Value is relative to zero.
 *  @return int the current datalement position 
 *
*/
   public int getDataElementPos()
     {
        return dataelementPos;
     }



/**
 *  returns a subfield tokeinzer
 *  @return IDataTokenizer
 *
*/
   public IDataTokenizer makeSubfieldTokenizer()
     {
        return new DataTokenizer(currentDataElement, new String(tokenGroups));
     }



  public class DataTokenizer implements IDataTokenizer
    {
/**
 *  similar to the java.lang.StringTokenizer.
 *  It seemed that StringTokenizer couldn't return a zero-length
 *  element between 2 tokens - such as token(*) and string in is (***) it would not find 2 
 *  zero-length tokens.
 *  @return boolean  data element ready?
 *
*/
      protected int currentPos;
      protected int stringLength;
      protected String tokenString;
      protected String tokens;
      protected char tokenBreak;

/**
 *  constructor
 *  @param inString - string to be tokenized
 *  @param inTokens - list of tokens
 *
*/
      public DataTokenizer(String inString, String inTokens)
       { 
         currentPos = -1;
         tokenString = inString;
         stringLength = tokenString.length();
         tokens = inTokens;
       }

/**
 *  to get next token
 *  @return String - the next token
 *
*/
      public String nextToken()
      {
       StringBuffer stringBuffer = new StringBuffer();
       if ( (currentPos > -1)
          && (currentPos < stringLength))
           tokenBreak = tokenString.charAt(currentPos);
       else tokenBreak = ' ';    
       currentPos++;
       int tokenPos;
       while ( (currentPos < stringLength)  &&  (tokens.indexOf(tokenString.charAt(currentPos)) < 0))
         {
           stringBuffer.append(tokenString.charAt(currentPos));
           currentPos++;
         }
       return stringBuffer.toString();
      }


        
/**
 *  indicates if there are more data elements to parse
 *  @return boolean  more to parse?
 *
*/
      public boolean hasMoreElements()
      {
        int i = currentPos;
        int j = stringLength;
        return (currentPos <= stringLength);
      }

/**
 *  how many tokens in list
 *  @return int - a count
 *
*/
      public int countTokens()
       {
         
         int pos = currentPos;
         if (pos == -1) 
           currentPos = 0;
         int cnt = 1;
         while ( (currentPos < stringLength) )
           {
             if (tokens.indexOf(tokenString.charAt(currentPos)) > -1)
                  cnt++;
             currentPos++;
           }
         currentPos = pos;  
         return cnt;
       }
    }
 }
