package com.americancoders.edi;

import java.util.*;
import java.io.*;
import org.w3c.dom.*;

/**
 *class for all EDI Transaction Sets
 *<p>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<br>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class TransactionSet extends Object implements Externalizable
 {
    
   

   String id = "";
   String name = "";
   String revision = "";
   String functionalGroup = "";
   String shortDescription = "";
   String xmlTag = "";
   public static final int XML_FORMAT = 1;
   public static final int X12_FORMAT = 2;
   public static final int EDIFACT_FORMAT = 3;
   public static final String X12_SEGMENT_DELIMITER = "\n";
   public static final String X12_FIELD_DELIMITER = "*";
   public static final String X12_GROUP_DELIMITER = "<";
   public static final String EDIFACT_SEGMENT_DELIMITER = "'";
   public static final String EDIFACT_FIELD_DELIMITER = "+";
   public static final String EDIFACT_GROUP_DELIMITER = ":";
   
   
   com.americancoders.edi.Table headerTable = null,
                                detailTable = null,
                                summaryTable = null;
/**
 * constructs a Transaction Set, used by Externalize function 
 */
 
public TransactionSet(){}      
/**
 * constructs a Transaction Set
 * <br>sets xml tag field to value in short description
 * @param String id
 * @param String revision number
 * @param String Functional Group
 * @param String short description
 *
*/ 

 public TransactionSet(String inId, String inName, String inRevision, String inFunctionalGroup, String inShortDescription)
   {
       setID(inId);
       setName(inName);
       setRevision(inRevision);
       setFunctionalGroup(inFunctionalGroup);
       setShortDescription(inShortDescription);
       setXMLTag(inShortDescription.replace(' ','_'));
   }
   
/**
 * constructs a Transaction Set
 * @param String id
 * @param String revision number
 * @param String Functional Group
 * @param String short description
 * @param String XML Tag
 *
*/ 

 public TransactionSet(String inId, String inName, String inRevision, String inFunctionalGroup, String inShortDescription, String inXMLTag)
   {
       setID(inId);
       setName(inName);
       setRevision(inRevision);
       setFunctionalGroup(inFunctionalGroup);
       setShortDescription(inShortDescription);
       setXMLTag(inXMLTag);
   }
   
 
/**
 * parses an EDI Document from tokenized string
 * 
 * @param TransactionTokenizedString  input string containing all of the transaction data pretokened by EDIObjects.Tokenizer
 *
 * @exception Exception 
 *                         thrown when the transaction id string is incorrect
 * @exception Exception 
 *                         thrown when an unknown segment id string is foundi
*/
public void parse(ITokenizer TransactionTokenizedString)
                throws Exception
   {
    if (headerTable != null)
             headerTable.parse(TransactionTokenizedString);
    if (detailTable != null)
             detailTable.parse(TransactionTokenizedString);
    if (summaryTable != null)
             summaryTable.parse(TransactionTokenizedString);
   }

/**
 * parses a XML EDI Document from a DOM node.
 * 
 * @param node  XML Node element
 *
 * @exception Exception 
 *                         thrown when the transaction id string is incorrect
 * @exception Exception 
 *                         thrown when an unknown segment id string is foundi
*/
public void parse(Node node)
                throws Exception
   {
    boolean headerDone = false;
    boolean detailDone = false;
    boolean summaryDone = false;
    Node cnode;
    NodeList nl = node.getChildNodes();
    int i;
	for (i = 0; i < nl.getLength(); i++)
	  {
	    
	    cnode = nl.item(i);  
	    if (cnode.getNodeType() != Node.ELEMENT_NODE) continue;
	   
        if (headerDone == false)
		  {
		     if (headerTable.doYouWantThisNode(cnode) == false)
			    headerDone = true;
		     else
			    {
				  headerTable.parse(cnode);
				  continue;
				}  
	      }			
        if (detailDone == false)
		  {
		     if (detailTable.doYouWantThisNode(cnode) == false)
			    detailDone = true;
		     else
			    {
				  detailTable.parse(cnode);
				  continue;
				}  
	      }			
        if (summaryDone == false)
		  {
		     if (summaryTable.doYouWantThisNode(cnode) == false)
			   throw new OBOEException("unknown node type found while parsing document");
		     else
			    {
				  summaryTable.parse(cnode);
				  continue;
				}  
	      }			
	  }		 
   }

/**
 * sets id for the Transaction Set
 * @param String id String
 *
*/
  public void setID(String inId){ id = inId; }
  
/**
 * sets name for the Transaction Set
 * @param String name String
 *
*/
  public void setName(String inName){ name = inName; }
  
  
/**
 * sets Revision for the Transaction Set
 * @param String Revision String
*
*/
 public void setRevision(String inRevision) {revision = inRevision;}
 
  
/**
 * sets Function Group for the Transaction Set
 * @param String Function Group String
 *
*/
 public void setFunctionalGroup(String inFunctionalGroup) {functionalGroup = inFunctionalGroup;}
 
/**
 * sets Short Description for the Transaction Set
 * @param String Description String
 *
*/
 public void setShortDescription(String inDesc) {shortDescription = inDesc;};

/**
 * sets header table for the Transaction Set
 * @param Table header table
 *
*/
public void setHeaderTable(Table inTable) {headerTable = inTable;}
 
/**
 * sets detail table for the Transaction Set
 * @param Table detail table
 *
*/
 public void setDetailTable(Table inTable) {detailTable = inTable;}
/**
 * sets summary table for the Transaction Set
 * @param Table summary table
 *
*/
 public void setSummaryTable(Table inTable) {summaryTable = inTable;}
 
/**
 * returns id for the Transaction Set
 * @return String
 *
 */
 
 public String getID(){ return id; }
 
/**
 * returns name for the Transaction Set
 * @return String
 *
 */
 
 public String getName(){ return name; }
 
  
/**
 * returns revision value for the Transaction Set
 * @return String
 *
*/

 public String getRevision() {return revision;}
  
 
/**
 * return Functional Group for the Transaction Set
 * @return String
 *
*/

 public String getFunctionalGroup() {return functionalGroup;}
 
 
/**
 * returns the Short Description for the Transaction Set
 * @return String
*/
 public String getShortDescription() {
    if (shortDescription == null || shortDescription.length() == 0)
       return id;
    return shortDescription;}
	
/**
 * returns header table for the Transaction Set
 * @return Table
 *
 */
 
 public Table getHeaderTable() {return headerTable;}
 
/**
 * returns detail table for the Transaction Set
 * @return Table
 *
 */
 
 public Table getDetailTable() {return detailTable;}
 
/**
 * returns summary table for the Transaction Set
 * @return Table
 *
 */
 
 public Table getSummaryTable() {return summaryTable;}
 
/**
 * returns all the data associated with the Transaction Set of type  Purchase_Order
 * @return String for all segments within the Transaction Set
 * @exception Exception - subsegments will throw
*/
public String get()
                throws Exception
   {
     StringBuffer outputTransactionSet = new StringBuffer();

     if (headerTable != null)
        outputTransactionSet.append(headerTable.get());
        
     if (detailTable != null)
        outputTransactionSet.append(detailTable.get());
        
     if (summaryTable != null)
        outputTransactionSet.append(summaryTable.get());
        
    return new String(outputTransactionSet);
   }

 /**
  * sets the xml tag field 
  * @param String tag value
  */
  
 
 public void  setXMLTag(String inXMLTag)
 {
    xmlTag = inXMLTag;
 }
 
 /**
  * returns the xml tag field 
  * @return String tag value
  */
  
  
 public String getXMLTag()
 {
    
    return xmlTag;
 }
 
 /**
 * validates segment syntax for correct DataElements
 * @exception Exception indicates why segment is invalid
 */
public void validate()
   throws Exception
   {
     if (headerTable != null)
        headerTable.validate();
        
     if (detailTable != null)
        detailTable.validate();
        
     if (summaryTable != null)
        summaryTable.validate();
        
   }


/**
 * returns the number of segments
 * @return int count
*/

public int getSegmentCount()
 {
     int segCount = 0;
     
     if (headerTable != null)
        segCount += headerTable.getSegmentCount();
        
     if (detailTable != null)
        segCount += detailTable.getSegmentCount();
        
     if (summaryTable != null)
        segCount += summaryTable.getSegmentCount();
     
     
     return segCount;
  }

/**
 * returns a formatted string of the transaction set
 * @param int type: TransactionSet.XML_FORMAT, TransactionSet.X12_FORMAT, TransactionSet.EDIFACT_FORMAT:
 * @return String formattedOutput
*/

public String getFormattedText(int formatType)
{
     StringBuffer sbFormattedText = new StringBuffer();
     switch (formatType)
       {
         case TransactionSet.XML_FORMAT:
            sbFormattedText.append('<' + getXMLTag() + ">\n");
            break;
         case TransactionSet.X12_FORMAT:
            break;
         case TransactionSet.EDIFACT_FORMAT:
            break;
         default:
           sbFormattedText.append("Transaction Set: " + getShortDescription() + "\n");
       }     
     if (headerTable != null)
        sbFormattedText.append(headerTable.getFormattedText(formatType));
        
     if (detailTable != null)
        sbFormattedText.append(detailTable.getFormattedText(formatType));
        
     if (summaryTable != null)
        sbFormattedText.append(summaryTable.getFormattedText(formatType));
        
     switch (formatType)
       {
         case TransactionSet.XML_FORMAT:
            sbFormattedText.append("</" + getXMLTag() + ">\n");
            break;
         case TransactionSet.X12_FORMAT:
            break;
         case TransactionSet.EDIFACT_FORMAT:
            break;
         default:
           sbFormattedText.append("Transaction Set: " + getShortDescription() + "\n");
       }     
        
    return new String(sbFormattedText);

 }

/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException - most likely class changed since written 
 * @exception ClassNotFoundException - only when dummy constructro not found
 */ 
 
   public void readExternal(ObjectInput in) 
     throws IOException, ClassNotFoundException
   {
    id = in.readUTF();
    name = in.readUTF();
    revision = in.readUTF();
    functionalGroup = in.readUTF();
    shortDescription = in.readUTF();
    xmlTag = in.readUTF();
    
    headerTable = (Table) in.readObject();
    detailTable = (Table) in.readObject();
    summaryTable = (Table) in.readObject();
   }
   
/** used by externalize methods
 * @param ObjectInput object input stream
 * @exception IOException 
 */ 
 
   public void writeExternal(ObjectOutput out)
     throws IOException
   {
    out.writeUTF(id);
    out.writeUTF(name);
    out.writeUTF(revision);
    out.writeUTF(functionalGroup);
    out.writeUTF(shortDescription);
    out.writeUTF(xmlTag);
    out.writeObject(headerTable);
    out.writeObject(detailTable);
    out.writeObject(summaryTable);
   }
   
 }
