package com.americancoders.edi;

import org.xml.sax.*;
import com.americancoders.edi.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.util.jar.*;

/**
 * class for building a Transaction Set from an edi xml file
 * <br>Class contains a main method to allow it to be invoked as an application.
 * <br>format: java com.americancoders.edi.TransactionSetFactory xmlfilename,
 * <br>where xmlfilename is a xml file based on transactionSet.dtd.
 *
 *<br>OBOE - Open Business Objects for EDI
 *<br>An EDI and XML Translator Written In Java 
 *<br>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<brp>All rights reserved
 *<br>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/

public class TransactionSetFactory extends HandlerBase 
   {
      protected int _iElement = 0;
      static protected TransactionSet currentTransactionSet = null;
      protected Table currentTable = null;
      protected ITemplateSegmentContainer currentTemplateSegmentContainer = null;
      protected TemplateSegment currentTemplateSegment = null;
      protected TemplateComposite currentCompositeDE = null;
      protected TemplateDE currentDataElement = null;
      protected String currentLoopID = "";
      protected String currentID = "";
     
      protected Stack currentSegContainerStack;
            
/**
 * construct the factory with a xml parser 
 * @param org.xml.sax.Parser parser
 *
*/

   public TransactionSetFactory()
   {
       currentTransactionSet = null;
       Stack currentSegContainerStack = new Stack();     
   }
   
   
/**
 * method called for each xml element found.
 * <br> process logic 
 * <ul>
 * <li> test each name found for edi type: transactionSet, table, segment, dataelement
 * <li> for each type pull appropriate attributes and construct object
 * <li> for transaction set build transaction set
 * <li> for table build a table 
 * <li> for segments build a template segment
 * <li> for data element build a template datalement
 * </li>
 * @param String data element name
 * @param AttributeList attributes associated with data element
 *
*/
   public void startElement(String name, AttributeList atts)
        throws SAXException
          {
            int i;
            _iElement++;
            if (name.compareTo("transactionSet") == 0)
              {
                String id="", 
                   tsName="",
                   revision="", 
                   functionalGroup="", 
                   description="", 
                   xmlTag="";
                for (i = 0; i < atts.getLength(); i++)
                 {
                   if (atts.getName(i).compareTo("id") == 0)
                       id = atts.getValue(i);
                   if (atts.getName(i).compareTo("name") == 0)
                       tsName = atts.getValue(i);
                   if (atts.getName(i).compareTo("revision") == 0)
                       revision = atts.getValue(i);
                   if (atts.getName(i).compareTo("functionalGroup") == 0)
                       functionalGroup = atts.getValue(i);
                   if (atts.getName(i).compareTo("description") == 0)
                     {
                       description = atts.getValue(i);
                       if (xmlTag == null)
                          xmlTag = description;
                     }     
                   if (atts.getName(i).compareTo("xmlTag") == 0)
                       xmlTag = atts.getValue(i);
                 }      
                if (currentTransactionSet == null) {
                   currentTransactionSet = new TransactionSet(id, tsName, revision, functionalGroup, description, xmlTag);
				   }
                else 
                      throw new SAXException("Transaction set already defined, see element " + _iElement);
                return;    
              }  
            if (name.compareTo("table") == 0)
              {
			   String xmlTag = "";
               currentSegContainerStack = new Stack();
               currentLoopID = "";
               for (i = 0; i < atts.getLength(); i++)
                 {
                   if (atts.getName(i).compareTo("xmlTag") == 0)
				      {
                       xmlTag = atts.getValue(i);
					   break;
					  } 
                 } 
               for (i = 0; i < atts.getLength(); i++)
                 {
                  if (atts.getName(i).compareTo("section") == 0) break;
                 } 
               if (i >= atts.getLength())
                   {
                      throw new SAXException("Table section name missing, see element " + _iElement);
                   }
               if (atts.getValue(i).compareTo("header") == 0)
                 {
                   if (currentTransactionSet.getHeaderTable() == null)
                     {
                      currentTable = new Table();
                      currentTransactionSet.setHeaderTable(currentTable);
                     } 
                   else
                      throw new SAXException("Header table already defined, see element " + _iElement);
                 }  
               if (atts.getValue(i).compareTo("detail") == 0)
                 {
                   if (currentTransactionSet.getDetailTable() == null)
                     {
                      currentTable = new Table();
                      currentTransactionSet.setDetailTable(currentTable);
                     } 
                   else
                      throw new SAXException("Detail table already defined, see element " + _iElement);
                 }  
               if (atts.getValue(i).compareTo("trailer") == 0)
                 {
                   if (currentTransactionSet.getSummaryTable() == null)
                     {
                      currentTable = new Table();
                      currentTransactionSet.setSummaryTable(currentTable);
                     } 
                   else
                      throw new SAXException("Summary section already defined, see element " + _iElement);
                 }  
                 currentTemplateSegmentContainer = currentTable;   
                 return;
              }  
            if (name.compareTo("segment") == 0)
              {
  			    String xmlTag = null;
                currentCompositeDE = null;
                int sequence = currentTemplateSegmentContainer.getTemplateSegmentSize();
                String description = "";
                String currentName = "";
                char required = 'O';
                int occurs = 1;
                for (i = 0; i < atts.getLength(); i++)
                 {
                  if (atts.getName(i).compareTo("id") == 0)
                    {
                        currentID = atts.getValue(i);
                    }    
                   if (atts.getName(i).compareTo("sequence") == 0)
                       sequence = Integer.parseInt(atts.getValue(i));
                   if (atts.getName(i).compareTo("description") == 0)
                      {
                        description = atts.getValue(i);
                        if (xmlTag == null)
                           xmlTag = description;
                      }     
                   if (atts.getName(i).compareTo("name") == 0)
                       currentName = atts.getValue(i);
                   if (atts.getName(i).compareTo("xmlTag") == 0)
                       xmlTag = atts.getValue(i);
                   if (atts.getName(i).compareTo("occurs") == 0)
                      {
                        String sOccurs = atts.getValue(i).trim();
                        if (sOccurs.length() > 0) // let it numeric parsing exception occur on its own
                            occurs = Integer.parseInt(sOccurs);
                      }      
                   if (atts.getName(i).compareTo("required") == 0)
                       required = atts.getValue(i).charAt(0);
                 }
                    
                currentTemplateSegment = new TemplateSegment(currentID, currentName, sequence , description, occurs, required, xmlTag);                
                   
                try {
                    currentSegContainerStack.push(currentTemplateSegmentContainer);
                    currentTemplateSegmentContainer.addTemplateSegment(currentTemplateSegment);
                    currentTemplateSegmentContainer = currentTemplateSegment;
                    }
                   catch (Exception e1)
                     { 
                        throw new SAXException(e1.getMessage()); 
                     }
                return;
                }
                
              
            if (name.compareTo("compositeDE") == 0)
              {
                String xmlTag = "";
                for (i = 0; i < atts.getLength(); i++)
                 {
                   if (atts.getName(i).compareTo("xmlTag") == 0)
                       xmlTag = atts.getValue(i);
				 }	   
                currentCompositeDE = setTemplateComposite(currentTemplateSegment.getTemplateDESize(), atts);
                currentTemplateSegment.addCompositeDE(currentCompositeDE);
                
                return;
              }
            if (name.compareTo("dataElement") == 0)
              {
                int pos;
                if (currentCompositeDE != null)
                  pos = currentCompositeDE.getTemplateDESize();
                else
                  pos = currentTemplateSegment.getTemplateDESize();
                currentDataElement =  setDataElement(pos, atts);
                if (currentCompositeDE != null)
                  currentCompositeDE.addDataElement(currentDataElement);
                else
                  currentTemplateSegment.addDataElement(currentDataElement);
                  
                return;      
              }       
              
              
            throw new SAXException("logic error: unknown type " + name + " " + currentID + " for element: " + _iElement);
            
        }
        
    /**
     * Method called by the SAX parser at the </ 
     *
     *
     * @param String name of element found
     *
     * @throws SAXException
     *
     *  
     */
    public void endElement(String name) throws SAXException {
            if (name.compareTo("transactionSet") == 0) {
                return;    
              }  
            if (name.compareTo("table") == 0)
              {
               currentTable.defineSegmentArray();
               return;                
              }
            if (name.compareTo("segment") == 0)
              {
                currentTemplateSegmentContainer = (ITemplateSegmentContainer) currentSegContainerStack.pop();
              }
            if (name.compareTo("compositeDE") == 0)
              {
                currentCompositeDE = null;
                
                return;
              }
            if (name.compareTo("dataElement") == 0)
              {
                currentDataElement = null;
                return;      
              }       
              
            //throw new SAXException("logic error: unknown type " + name + " for element: " + _iElement);
    } 
        
        
		
/**
 * help method to build a template composite
 * @param AttributeList attributes associated with data element
 * @return TemplateComposite template composite
 *
*/
    public TemplateComposite setTemplateComposite(int pos, AttributeList atts)
          throws SAXException
      {
        int i;
        String id = "",
               thisName = "",
               description="";
        int occurs = 1;
        int sequence = pos+1;
  	    String xmlTag = "";
        for (i = 0; i < atts.getLength(); i++)
           {
            if  (atts.getName(i).compareTo("id") == 0) 
               id = atts.getValue(i);
            if  (atts.getName(i).compareTo("description") == 0) 
               {
                description = atts.getValue(i);
                if (xmlTag == null)
                   xmlTag = description;
               }    
            if  (atts.getName(i).compareTo("name") == 0) 
               thisName = atts.getValue(i);
            if  (atts.getName(i).compareTo("sequence") == 0) 
               sequence = Integer.parseInt(atts.getValue(i));
            if  (atts.getName(i).compareTo("xmlTag") == 0) 
               xmlTag = atts.getValue(i); 
           }    
         return new TemplateComposite(id, thisName, sequence, description, xmlTag);
      } 
      
      
/**
 * help method to build a template data element
 * @param int pos in segment or composite
 * @param AttributeList attributes associated with data element
 * @return TemplateDE template data element
 *
*/
    public TemplateDE setDataElement(int pos, AttributeList atts)
          throws SAXException
      {
        int i;
        String id = "", thisName="", type = "", description="";
        int minLength = 0, maxLength = 0; 
  	    String xmlTag = "", required= "";
  	    int sequence = pos+1;
        for (i = 0; i < atts.getLength(); i++)
           {
            if  (atts.getName(i).compareTo("type") == 0) 
               type = atts.getValue(i);
            if  (atts.getName(i).compareTo("required") == 0) 
               required = atts.getValue(i);
            if  (atts.getName(i).compareTo("id") == 0) 
               id = atts.getValue(i);
            if  (atts.getName(i).compareTo("name") == 0) 
               thisName = atts.getValue(i);
            if  (atts.getName(i).compareTo("description") == 0) 
               {
                description = atts.getValue(i);
                if (xmlTag == null)
                   xmlTag = description;
               }    
            if  (atts.getName(i).compareTo("sequence") == 0) 
               sequence = Integer.parseInt(atts.getValue(i));
            if  (atts.getName(i).compareTo("minLength") == 0) 
               minLength = Integer.parseInt(atts.getValue(i));
            if  (atts.getName(i).compareTo("maxLength") == 0) 
               maxLength = Integer.parseInt(atts.getValue(i)); 
            if  (atts.getName(i).compareTo("xmlTag") == 0) {
               xmlTag = atts.getValue(i); 
            }  
               
           }    
         return new TemplateDE(id, thisName, sequence, type, required.charAt(0), description, minLength, maxLength, xmlTag);
      } 
      
      
		
 /**
 * method called when xml file is read to completion
 *
 */
   public void endDocument() {
//        System.out.println("Doc contains " + _iElement + " elements ");
      }
	  
 /**
 * catches warning SAXParseExceptions
 * this code sends exception to stdio and allows class to continue
 */
    public void warning(SAXParseException e) throws SAXException {
        System.err.println("Warning at (file " + e.getSystemId() + ", line " +
            e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
    }
	
 /**
 * catches error SAXParseExceptions
 * this code causes exception to continue
 */
    public void error(SAXParseException e) throws SAXException {
        throw new SAXException("Error at (file " + e.getSystemId() + ", line " +
            e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
    }
	
 /**
 * catches fatal SAXParseExceptions
 * this code causes exception to continue
 */
    public void fatalError(SAXParseException e) throws SAXException {
        throw new SAXException("Fatal Error at (file " + e.getSystemId() +
            ", line " + e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
    }
    
	
 /**
 * static class method will build a transaction set based on input string and OBOE.properties defintion
 * see OBOE.properties file to define the directory path
 * file name is preppended with xmlPath from OBOE.properties and appended with ".xml"
 * <br> example
 * <br> OBOE.properties file contains: xmlPath = c:/xmlDefinitions/
 * <br> input String is 840
 * <br> method will read file named: x:/xmlDefinitions/840.xml
 * <br> method will search for the OBOE.properties file 
 * <ul> <li>local directory <li> user.home directory <li> java.home directory <li> </ul>
 * @param String transaction id
 * @return TransactionSet 
 */
public static TransactionSet buildTransactionSet(String inTSID)
    throws Exception
   {
        String OBOEPropertiesFileName = "OBOE.properties";
        File f3, f2, f1 = new File(OBOEPropertiesFileName);
        if (f1.exists() == false)
          {
            OBOEPropertiesFileName = System.getProperty("user.home") + "/OBOE.properties";
            f2 = new File(OBOEPropertiesFileName);
            if (f2.exists() == false)
              {
                OBOEPropertiesFileName = System.getProperty("java.home") + "/OBOE.properties";
                f3 = new File(OBOEPropertiesFileName);
                if (f3.exists() == false)
                  {
                    System.err.println("Searched for OBOE.properties as " + f1.getAbsolutePath());
                    System.err.println("Searched for OBOE.properties as " + f2.getAbsolutePath());
                    System.err.println("Searched for OBOE.properties as " + f3.getAbsolutePath());
                    throw new OBOEException("OBOE.properties file not found");
                  }
              }
          }    
        return buildTransactionSet(inTSID, OBOEPropertiesFileName);
        
     //return null;
   } 
   
 /**
 * static class method will build a transaction set based on input string and OBOE.properties definition
 * see OBOE.properties file to define the directory path
 * file name is preppended with xmlPath from OBOE.properties and appended with ".xml"
 * <br> example
 * <br> OBOE.properties file contains: xmlPath = c:/xmlDefinitions/
 * <br> input String is 840
 * <br> method will read file named: x:/xmlDefinitions/840.xml
 * @param String qualified properties files name
 * @param String transaction id
 * @return TransactionSet 
 */
public static TransactionSet buildTransactionSet(String inTSID, String inPropertiesFileName)
    throws Exception
   {
        String xmlPath, objectPath;
        File xmlFile, objectFile;
        boolean useXML = false;
        try {
        Properties properties = new Properties();
        FileInputStream inStream = new FileInputStream(inPropertiesFileName);
        properties.load(inStream);
        String path = properties.getProperty("xmlPath");
        
        xmlPath = path + inTSID + ".xml"; 
        objectPath = path + inTSID + ".object"; 
        
        xmlFile = new File(xmlPath);
        objectFile = new File(objectPath);
        
        if (xmlFile.exists() == false)
           throw new FileNotFoundException(xmlPath);
        else   
        if (objectFile.exists() == false)
           useXML = true;
        else if (xmlFile.lastModified() > objectFile.lastModified())
           useXML = true;
         try { 
         if (useXML == false)  
         {  
             FileInputStream fis = new FileInputStream(objectFile);
             ObjectInputStream ios = new ObjectInputStream(fis);
             currentTransactionSet = (TransactionSet) ios.readObject();
             fis.close();
             return currentTransactionSet;
         }
         }
         catch (IOException ioe) { useXML = true; }
         catch (Exception ex) {
            System.out.println("message" + ex.getMessage());
            throw ex;
        }

        org.xml.sax.Parser parser = org.xml.sax.helpers.ParserFactory.makeParser("org.apache.xerces.parsers.SAXParser");

        TransactionSetFactory tsf = new TransactionSetFactory();
        parser.setDocumentHandler(tsf);
        parser.setErrorHandler(tsf);
        InputSource is = new InputSource("file:///"+xmlPath);
        parser.parse(is);       
        FileOutputStream fos = new FileOutputStream(objectPath);
        ObjectOutputStream oos = new ObjectOutputStream(fos);
        oos.writeObject(currentTransactionSet);
        oos.close();
        return currentTransactionSet;
           
        } catch (Exception ex) {
            System.out.println("message" + ex.getMessage());
            throw ex;
        }
     
   } 
      
 /**
 * static class method will build a transaction set based on url file name and OBOE.properties defintion
 * used by the sample html viewer on web page
 * see OBOE.properties file to define the directory path
 * file name is preppended with xmlPath from OBOE.properties and appended with ".xml"
 * <br> example
 * <br> OBOE.properties file contains: xmlPath = c:/xmlDefinitions/
 * <br> input String is 840
 * <br> method will read file named: x:/xmlDefinitions/840.xml
 * @param String transaction id
 * @param URL url of properties file
 * @return TransactionSet 
 */
public static TransactionSet buildTransactionSet(String inTSID, URL inURL)
     throws Exception
   {
        try {
                      
        URL url = new URL(inURL + inTSID + ".xml");
        org.xml.sax.Parser parser = (org.xml.sax.Parser)Class.forName("com.ibm.xml.parsers.ValidatingSAXParser").newInstance();       
        TransactionSetFactory tsf = new TransactionSetFactory();
        parser.setDocumentHandler(tsf);
        parser.setErrorHandler(tsf);
        
        parser.parse(url.toString());
        return currentTransactionSet;
           
        } catch (Exception ex) {
            System.out.println("message" + ex.getMessage());
            //ex.printStackTrace();
            throw ex;
        }
   } 
   
   
/**
 * static main class used for testing
 * <br>Class contains a main method to allow it to invoked as an application.
 * <br>format: java com.americancoders.edi.TransactionSetFactory xmlfilename
 * <br>where xmlfilename is a xml file based on transactionSet.dtd.
*/
	  
public static void main(String args[])
    {
        if (args.length < 1 || args.length > 2)
          {
            System.out.println("java TransactionSetFactor xxx xxx");
            System.exit(0);
          }
          
        if (args.length != 1) 
          {
            try {
                      
             org.xml.sax.Parser parser = (org.xml.sax.Parser)Class.forName("com.ibm.xml.parsers.ValidatingSAXParser").newInstance();       
             TransactionSetFactory tsf = new TransactionSetFactory();
             parser.setDocumentHandler(tsf);
             parser.setErrorHandler(tsf);
             parser.parse(args[0]);
             System.out.println(currentTransactionSet.get());
           
            } catch (Exception ex) {
                System.out.println("message" + ex.getMessage());
                ex.printStackTrace();
            }
          }
            
    }
    
   
} 
