package com.americancoders.edi.x12;

/**
 *OBOE - Open Business Objects for EDI
 *<P>java access to EDI 
 *<p>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<p>All rights reserved
 *<p>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1. 
*/

import java.util.*;
import java.lang.*;
import com.americancoders.edi.*;
import com.americancoders.edi.x12.*;
import com.americancoders.util.*;

/**
 * class for wrapping an EDI transaction set within an EDI Envelope
 * <br> <I> can handle only on transaction set </I>
 * <BR> look here to see how to build an EDI Object
 *
*/

public class Envelope
 {

    Segment ISA_Interchange_Control_Header,
        GS_Functional_Group_Header,
        GE_Functional_Group_Trailer,
        IEA_Interchange_Control_Trailer;


    StringBuffer  outputTransactionSet;

/**
 * instantiates the class from all related ediObjects class
 * @exception Exception - can occur when a segment or dataelement class is not correct or fully populated
 */

   public Envelope()
           throws Exception
      {

        
        ISA_Interchange_Control_Header = Interchange_Control_Header.getInstance();
        GS_Functional_Group_Header = Functional_Group_Header.getInstance();
        GE_Functional_Group_Trailer = Functional_Group_Trailer.getInstance();
        IEA_Interchange_Control_Trailer = Interchange_Control_Trailer.getInstance();
        outputTransactionSet = new StringBuffer();


      }


/**
 * returns the EDI (x12) formatted document in a String
 * @return String the formatted document
 */

    public String getOutputTransactionSet(TransactionSet inputTransactionSet)
      throws Exception
      {
        String saveDt = Util.currentDate();
        String saveTm = Util.currentTime();
       
        ISA_Interchange_Control_Header.getDataElement(0).set("00");
        ISA_Interchange_Control_Header.getDataElement(1).set("        ");
        ISA_Interchange_Control_Header.getDataElement(2).set("00");
        ISA_Interchange_Control_Header.getDataElement(3).set("        ");
        ISA_Interchange_Control_Header.getDataElement(4).set("01");
        ISA_Interchange_Control_Header.getDataElement(5).set("111111");
        ISA_Interchange_Control_Header.getDataElement(6).set("01");
        ISA_Interchange_Control_Header.getDataElement(7).set("222222");
        ISA_Interchange_Control_Header.getDataElement(8).set(saveDt);
        ISA_Interchange_Control_Header.getDataElement(9).set(saveTm);
        ISA_Interchange_Control_Header.getDataElement(10).set("U");
        ISA_Interchange_Control_Header.getDataElement(11).set("003090");
        ISA_Interchange_Control_Header.getDataElement(12).set("00000001");
        ISA_Interchange_Control_Header.getDataElement(13).set("0");
        ISA_Interchange_Control_Header.getDataElement(14).set("T");
        ISA_Interchange_Control_Header.getDataElement(15).set(String.valueOf((char) 0x1f));
        outputTransactionSet.append(ISA_Interchange_Control_Header.get());
         
        GS_Functional_Group_Header.getDataElement(0).set(inputTransactionSet.getFunctionalGroup());
        GS_Functional_Group_Header.getDataElement(1).set("111111");
        GS_Functional_Group_Header.getDataElement(2).set("222222");
        GS_Functional_Group_Header.getDataElement(3).set(saveDt);
        GS_Functional_Group_Header.getDataElement(4).set(saveTm);
        GS_Functional_Group_Header.getDataElement(5).set("000000");
        GS_Functional_Group_Header.getDataElement(6).set("X");
        GS_Functional_Group_Header.getDataElement(7).set(inputTransactionSet.getRevision());
        outputTransactionSet.append(GS_Functional_Group_Header.get());
        
        outputTransactionSet.append(inputTransactionSet.get());
        
        GE_Functional_Group_Trailer.getDataElement(0).set("1");
        GE_Functional_Group_Trailer.getDataElement(1).set(GS_Functional_Group_Header.getDataElement(6).get());
        outputTransactionSet.append(GE_Functional_Group_Trailer.get());
        
        IEA_Interchange_Control_Trailer.getDataElement(0).set(Integer.toString(inputTransactionSet.getSegmentCount()));
        IEA_Interchange_Control_Trailer.getDataElement(1).set(ISA_Interchange_Control_Header.getDataElement(13).get());
        outputTransactionSet.append(IEA_Interchange_Control_Trailer.get());
        
        return new String(outputTransactionSet);
      }
      
      
  /** returns the Interchange Control Header build for the envelope
   *  @return Segment
   */ 
   
  public Segment getInterchange_Control_Header() {return ISA_Interchange_Control_Header;}
  
  
  
  /** returns the Functional Group Header build for the envelope
   *  @return Segment
   */ 
   
  public Segment getFunctional_Group_Header() {return GS_Functional_Group_Header;}
  
  
  
  /** returns the Functional Group Trailer build for the envelope
   *  @return Segment
   */ 
   
  public Segment getFunctional_Group_Trailer() {return GE_Functional_Group_Trailer;}
  
  
  
  /** returns the Interchange Control Trailer build for the envelope
   *  @return Segment
   */ 
   
  public Segment getInterchange_Control_Trailer() {return IEA_Interchange_Control_Trailer;}
  
 }
