package com.americancoders.edi.x12;
/**
 *OBOE - Open Business Objects for EDI
 *<P>java access to EDI 
 *<p>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<p>All rights reserved
 *<p>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1. 
*/

import com.americancoders.edi.*;

public class Functional_Acknowledgment
  {
  
  /**
   * Static class method used to generate a 997 transactionSet.  It always returns a positive 
   *  acknowledgment and assumes it is reponding to 1 transaction set.
   *  @param TransactionSet ts we're responding to
   *  @param Segment ISA segment used for control numbers to indicate what we are responding to
   *  @return TransactionSet - functional acknowledgement transaction set
   *  @exception Exception - for instance using FA to respond to a FA is not correct.
   */
	 
	 
  public static TransactionSet PostiveAcknowledgment(TransactionSet tsRespondingTo, Segment ISASegment)
                     throws Exception
	  {	    
	    
	        if (tsRespondingTo.getID().compareTo("997") == 0)
	           throw new OBOEException("Functional Acknowledgment should not be used to respond to another Functional Acknowledgment");
	            
 	 	    Table responseTable, currentTable;
	 	    Segment responseSegment, currentSegment;
	 	    DataElement currentDE;
	 	    
	 	    responseTable = tsRespondingTo.getHeaderTable();
	 	    responseSegment = responseTable.getSegment("ST");
	 	    
	        TransactionSet transactionSet = TransactionSetFactory.buildTransactionSet("997");
	  
	        currentTable = transactionSet.getHeaderTable();
	  
	        currentSegment = currentTable.createSegment("ST");
	        currentTable.addSegment(currentSegment);
	        currentDE = currentSegment.getDataElement(1);
	        currentDE.set("997");
	        currentDE = currentSegment.getDataElement(2);
	        currentDE.set(responseSegment.getDataElement(2).get());	        
	    
	  
	        currentSegment = currentTable.createSegment("AK1");
	        currentTable.addSegment(currentSegment);
	        currentDE = currentSegment.getDataElement(1);
	        currentDE.set(tsRespondingTo.getFunctionalGroup());
	        currentDE = currentSegment.getDataElement(2);
	        currentDE.set(ISASegment.getDataElement("ISA13").get());
 
            currentSegment = currentTable.createSegment("AK9");
            currentTable.addSegment(currentSegment);
            currentDE = currentSegment.getDataElement(1);
            currentDE.set("A");
            currentDE = currentSegment.getDataElement(2);
            currentDE.set("1");
            currentDE = currentSegment.getDataElement(3);
            currentDE.set("1");
			
            currentSegment = currentTable.createSegment("SE");
            currentTable.addSegment(currentSegment);
            currentDE = currentSegment.getDataElement(1);
            currentDE.set("4");
            currentDE = currentSegment.getDataElement(2);
            currentDE.set(responseSegment.getDataElement(2).get());
			
			return transactionSet;
       }
	 
  }
