package com.americancoders.edi.x12;

/**
 *OBOE - Open Business Objects for EDI
 *<P>java access to EDI 
 *<p>Copyright 1998-2000 - American Coders, LTD  - Raleigh NC USA
 *<p>All rights reserved
 *<p>American Coders, Ltd
 *<br>P. O. Box 97462
 *<br>Raleigh, NC  27624  USA
 *<br>1-919-846-2014
 *<br>http://www.americancoders.com
@author Joe McVerry, American Coders Ltd.
@version 1.4.1
*/


import java.util.*;
import java.io.*;
import java.net.*;

import com.americancoders.edi.*;
import com.americancoders.edi.x12.*;


/**
 * class to parse input string for all defined ediObjects Transaction Sets
 * <br> x12 dependent
 *
*/

public class Parser 
 {
       URL url=null;
       Segment ISA_Interchange_Control_Header,  
               ISB_Grade_of_Service_Request=null,// not required
               ISE_Deferred_Delivery_Request=null,//not required
               TA1_Interchange_Acknowledgment[]=null,//not required
               GS_Functional_Group_Header, 
               GE_Functional_Group_Trailer, 
               IEA_Interchange_Control_Trailer;

       TransactionSet currentTransactionSet = null;
	   int positionOfCurrentTransactionSet = -1;
	   Vector transactionSetList = new Vector();
/**
 * create a transaction set from input string 
 * @param String the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(String inString)
      throws Exception
    {
        ParseString(inString);
    }

/**
 * builds  a transaction set by reading a file
 * @param File the filename containing the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(File inFile)
      throws Exception
    {
         byte b[] = new byte[(int) inFile.length()];
         FileInputStream fis = new FileInputStream(inFile);
         fis.read(b);
         ParseString(new String(b));
    }
/**
 * builds  a transaction set by reading a file from a url
 * @param URL the URL containing the edi document 
 * @param String input text
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(URL inURL, String text)
      throws Exception
    {
         url = inURL;            
         ParseString(text);
    }


/**
 * builds  a parser from an existing transactionSet object
 * @param TransactionSet the filename containing the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public Parser(TransactionSet inTransactionSet)
      throws Exception
    {
         currentTransactionSet = inTransactionSet;            
    }


/**
 * builds  a transaction set from input string 
 * @param String the edi document 
 * @exception Exception
 *                      - unknown transaction set, this transaction set is undefined to ediObjects
*/

  public void ParseString(String inString)
      throws Exception
    {
       int segCount; 
       int deCount;

       int posStart; // = inString.indexOf("\nISA");
       // if (posStart < 0)
          posStart = inString.indexOf("ISA");
       if (posStart < 0)
          throw new OBOEException("ISA segment missing");
       int posStop = inString.indexOf("IEA");
       if (posStop < 0)
          throw new OBOEException("IEA segment missing");
       //posStop = posStop + inString.substring(posStop).indexOf("\n");

       Tokenizer et = new Tokenizer(inString.substring(posStart), "X12");  // , posStop-1));
       segCount = et.countSegments();

       et.getNextSegment();
       String findID =  et.getNextDataElement();
       if (findID.compareTo(Interchange_Control_Header.id) != 0)
          throw new OBOEException("Segment ID " + findID + " found, was expecting " + Interchange_Control_Header.id + ".");
       ISA_Interchange_Control_Header = Interchange_Control_Header.getInstance();
       ISA_Interchange_Control_Header.parse(et);
               
       findID =  et.getCurrentDataElement();
       if (findID.compareTo(Grade_of_Service_Request.id) == 0)
          {
            ISB_Grade_of_Service_Request = Grade_of_Service_Request.getInstance();
            ISB_Grade_of_Service_Request.parse(et);   
            findID =  et.getCurrentDataElement();
          }  
       if (findID.compareTo(Deferred_Delivery_Request.id) == 0)
          {
            ISE_Deferred_Delivery_Request = Deferred_Delivery_Request.getInstance();
            ISE_Deferred_Delivery_Request.parse(et);   
            findID =  et.getCurrentDataElement();
          }  
       if (findID.compareTo(Interchange_Acknowledgment.id) == 0)
          {
            Vector v_Interchange_Acknowledgment = new Vector();
            do {
                Segment Temp_Interchange_Acknowledgment = Interchange_Acknowledgment.getInstance();
                Temp_Interchange_Acknowledgment.parse(et);   
                v_Interchange_Acknowledgment.addElement(Temp_Interchange_Acknowledgment);
                findID =  et.getCurrentDataElement();
            } while (findID.compareTo(Interchange_Acknowledgment.id) == 0);
            TA1_Interchange_Acknowledgment = new Segment[v_Interchange_Acknowledgment.size()];
            for (int i = 0; i < v_Interchange_Acknowledgment.size(); i++)
               TA1_Interchange_Acknowledgment[i] = (Segment) v_Interchange_Acknowledgment.elementAt(i);
          }  
       

       findID =  et.getCurrentDataElement();
	   while  (findID.compareTo(Functional_Group_Header.id) == 0)
	     {
           GS_Functional_Group_Header = Functional_Group_Header.getInstance();
           GS_Functional_Group_Header.parse(et);
           
           findID =  et.getCurrentDataElement();
    
           while (findID.compareTo("ST") == 0)
           {
               findID = et.getNextDataElement();
               et.resetSegment();               
               if (currentTransactionSet == null)
                {
                 if (url != null)
                  currentTransactionSet = TransactionSetFactory.buildTransactionSet(findID, url);
                 else   
                  currentTransactionSet = TransactionSetFactory.buildTransactionSet(findID);
                }                 
               currentTransactionSet.parse(et);                
			   transactionSetList.addElement(currentTransactionSet);
               findID =  et.getCurrentDataElement();
           }
           if (findID.compareTo(Functional_Group_Trailer.id) != 0)
              throw new OBOEException("Segment ID " + findID + " found, was expecting " + Functional_Group_Trailer.id + ".");
           GE_Functional_Group_Trailer =  Functional_Group_Trailer.getInstance();
           GE_Functional_Group_Trailer.parse(et);
           findID =  et.getCurrentDataElement();
	   }

       if (findID.compareTo(Interchange_Control_Trailer.id) != 0)
          throw new OBOEException("Segment ID " + findID + " found, was expecting " + Interchange_Control_Trailer.id + ".");
       IEA_Interchange_Control_Trailer = Interchange_Control_Trailer.getInstance();
       IEA_Interchange_Control_Trailer.parse(et);
	   currentTransactionSet = (TransactionSet) transactionSetList.elementAt(0);

    }


/**
 * returns the transaction set that was parsed 
 * @return TransactionSet - the transaction set parsed when object was built.
*/
  public TransactionSet getTransactionSet()
    {
        return currentTransactionSet;
    }

/**
 * returns the transaction set at a specified postion
 * @return TransactionSet - the transaction set parsed when object was built.
 * @param int - position of transaction set
 & @exception - vector out of bounds exception most likely will occur
*/
  public TransactionSet getTransactionSet(int pos)
     throws Exception
    {
	    positionOfCurrentTransactionSet++;
		currentTransactionSet = (TransactionSet) transactionSetList.elementAt(pos);
        return currentTransactionSet;
    }

/**
 * returns the next transaction set that was parsed 
 * @return TransactionSet - the transaction set parsed when object was built.
 & @exception - vector out of bounds exception most likely will occur
*/
  public TransactionSet getNextTransactionSet()
     throws Exception
    {
	    positionOfCurrentTransactionSet++;
		currentTransactionSet = (TransactionSet) transactionSetList.elementAt(positionOfCurrentTransactionSet);
        return currentTransactionSet;
    }

/**
 * returns the number of transaction sets that were parsed 
 * @return int
 */
  public int getTransactionSetCount()
    {
	    return transactionSetList.size();
    }

  
  /** returns the parsed Interchange Control Header
   *  @return Segment
   */ 
   
  public Segment getInterchange_Control_Header() {return ISA_Interchange_Control_Header;}
  
  
  /** returns the parsed Functional Group Header
   *  @return Segment
   */ 
   
  public Segment getFunctional_Group_Header() {return GS_Functional_Group_Header;}
  
  
  /** returns the parsed Functional Group Trailer
   *  @return Segment
   */ 
   
  public Segment getFunctional_Group_Trailer() {return GE_Functional_Group_Trailer;}
  
  
      
  /** returns the parsed Interchange Trailer Header
   *  @return Segment
   */ 
   
  public Segment getInterchange_Control_Trailer() {return IEA_Interchange_Control_Trailer;}
  
  
 }
