package com.americancoders.util;

import java.io.*;
import javax.activation.*;



/* This class implements a typed DataSource from :
 * 	an InputStream
 *	a byte array
 * 	a String
 */
public class ByteArrayDataSource implements DataSource {
    private byte[] data; // data
    private String type; // content-type

    /* Create a datasource from an input stream */
    public ByteArrayDataSource(InputStream is, String type) {
        this.type = type;
        try { 
            ByteArrayOutputStream os = new ByteArrayOutputStream();
	    int ch;

	    while ((ch = is.read()) != -1)
                // XXX : must be made more efficient by
	        // doing buffered reads, rather than one byte reads
	        os.write(ch);
	    data = os.toByteArray();

        } catch (IOException ioex) { }
    }

    /* Create a datasource from a byte array */
    public ByteArrayDataSource(byte[] data, String type) {
        this.data = data;
	this.type = type;
    }

    /* Create a datasource from a String */
    public ByteArrayDataSource(String data, String type) {
	try {
	    // Assumption that the string contains only ascii
	    // characters ! Else just pass in a charset into this
	    // constructor and use it in getBytes()
	    this.data = data.getBytes("iso-8859-1");
	} catch (UnsupportedEncodingException uex) { }
	this.type = type;
    }

    /** 
     * taken from sun's exmaples
     * @return InputStream
     * @exception IOException from sun examples
     */     
     
    public InputStream getInputStream() throws IOException {
	if (data == null)
	    throw new IOException("no data");
	return new ByteArrayInputStream(data);
    }


    /** 
     * taken from sun's exmaples
     * @return OutputStream
     * @exception IOException from sun examples
     */     
     
     public OutputStream getOutputStream() throws IOException {
     	return new ByteArrayOutputStream();
    }

    /** 
     * returns data in byte array
     * @return byte[]
     */ 
    public byte[] getData() {
        return data;
    }

    /** 
     * taken from sun's exmaples
     * @return String type
     */ 
    public String getContentType() {
        return type;
    }

    /** 
     * always returns edix12
     * @return String constant "edix12"
     */ 
     
    public String getName() {
        return "edix12";
    }
    
}
